@@ -60,11 +60,11 @@ end
 
 function Python.expandVar(self, k, v, vType)
    local lineA = {}
-
+   v                 = tostring(v):multiEscaped()
    lineA[#lineA + 1] = 'os.environ['
    lineA[#lineA + 1] = k:doubleQuoteString()
    lineA[#lineA + 1] = '] = '
-   lineA[#lineA + 1] = v:doubleQuoteString()
+   lineA[#lineA + 1] = v
    lineA[#lineA + 1] = ';\n'
    local line        = concatTbl(lineA,"")
    stdout:write(line)
