@@ -42,8 +42,6 @@ require("strict")
 local MC_MgrLoad       = inheritsFrom(MasterControl)
 local M                = MC_MgrLoad
 local dbg              = require("Dbg"):dbg()
-local format           = string.format
-local getenv           = os.getenv
 M.my_name              = "MC_MgrLoad"
 M.my_sType             = "load"
 M.my_tcl_mode          = "load"
