@@ -65,6 +65,7 @@ require("strict")
 require("utils")
 require("inherits")
 
+_G._DEBUG         = false   -- Required by the new lua posix
 local M           = {}
 local MT          = require("MT")
 local dbg         = require("Dbg"):dbg()
@@ -73,6 +74,8 @@ local huge        = math.huge
 local pack        = (_VERSION == "Lua 5.1") and argsPack or table.pack
 local posix       = require("posix")
 local sort        = table.sort
+local malias      = require("MAlias"):build()
+local concatTbl   = table.concat
 MName             = M
 --------------------------------------------------------------------------
 -- This function allows for taking the name and remove one
@@ -175,8 +178,10 @@ function M.new(self, sType, name, action, is, ie)
       local Match    = require("MN_Match")
       local Latest   = require("MN_Latest")
       local Between  = require("MN_Between")
+      local Exact    = require("MN_Exact")
 
-      local findT   = {}
+      local findT      = {}
+      findT["exact"]   = Exact
       findT["match"]   = Match
       findT["latest"]  = Latest
       findT["between"] = Between
@@ -184,8 +189,12 @@ function M.new(self, sType, name, action, is, ie)
       s_findT          = findT
    end
 
+   local default_action = (LMOD_EXACT_MATCH == "yes") and "exact" or "match"
+
+
+
    if (not action) then
-      action = masterTbl().latest and "latest" or "match"
+      action = masterTbl().latest and "latest" or default_action
    end
    local o = s_findT[action]:create()
 
@@ -198,12 +207,11 @@ function M.new(self, sType, name, action, is, ie)
       local t = name
       o._name = t.userName
    else
-      name    = (name or ""):gsub("/+$","")  -- remove any trailing '/'
-      o._name = name
+      o._name = (name or ""):trim():gsub("/+$","")  -- remove any trailing '/'
    end
    o._action   = action
    o._is       = is or ''
-   o._ie       = ie or tostring(1234567890)
+   o._ie       = ie or tostring(99999999)
    o._range    = {}
    o._range[1] = is
    o._range[2] = ie -- This can be nil and that is O.K.
@@ -225,7 +233,7 @@ function M.buildA(self,sType, ...)
    for i = 1, arg.n do
       local v = arg[i]
       if (type(v) == "string" ) then
-         a[#a + 1] = self:new(sType, v)
+         a[#a + 1] = self:new(sType, v:trim())
       elseif (type(v) == "table") then
          a[#a + 1] = v
       end
@@ -272,8 +280,14 @@ local function lazyEval(self)
    end
 
    local mt   = MT:mt()
+
+   ------------------------------------------------------------------------
+   -- Must resolve user name which might be an alias to a real module name
+
    local name = self._name
    if (sType == "load") then
+      self._name = malias:resolve(self._name)
+      name       = self._name
       for level = 0, 1 do
          local n = shorten(name, level)
          if (mt:locationTbl(n)) then
@@ -330,7 +344,7 @@ function M.version(self)
    dbg.print{"sn:      ", self._sn,"\n"}
    dbg.print{"name:    ", self._name,"\n"}
    dbg.print{"version: ", self._version,"\n"}
-      
+
    if ((self._sn and self._sn == self._name) and
        (self._sType == "load" or self._sType == "userName")) then
       dbg.fini("MName:version")
@@ -343,53 +357,11 @@ function M.version(self)
    return self._version
 end
 
---------------------------------------------------------------------------
--- This is used to find a default file that maybe in symbolic link chain. 
--- @param path a file path.
--- @return This returns the absolute path.
-local function followDefault(path)
-   if (path == nil) then return nil end
-   dbg.start{"followDefault(path=\"",path,"\")"}
-   local attr      = lfs.symlinkattributes(path)
-   local result    = path
-   local accept_fn = accept_fn
-
-   if (attr == nil) then
-      result = nil
-   elseif (attr.mode == "link") then
-      local rl = posix.readlink(path)
-      local a  = {}
-      local n  = 0
-      for s in path:split("/") do
-         n = n + 1
-         a[n] = s or ""
-      end
-
-      a[n] = ""
-      local i  = n
-      for s in rl:split("/") do
-         if (s == "..") then
-            i = i - 1
-         else
-            a[i] = s
-            i    = i + 1
-         end
-      end
-      result = concatTbl(a,"/")
-   end
-   dbg.print{"result: ",result,"\n"}
-   dbg.fini("followDefault")
-   if (not accept_fn(result)) then
-      result = false
-   end
-   return result
-end
-
 --------------------------------------------------------------------------
 -- Build the initial table for reporting a module file location.
 -- @return the initial table.
 local function module_locationT()
-   return { fn = nil, modFullName = nil, modName = nil, default = 0}
+   return { fn = nil, modFullName = nil, modName = nil, version = nil, default = 0}
 end
 
 --------------------------------------------------------------------------
@@ -406,45 +378,31 @@ function M.find_exact_match(self, pathA)
    local found      = false
    local result     = nil
    local fullName   = ""
-   local modName    = ""
    local sn         = self:sn()
-   local searchExtT = accept_extT()
-   local numExts    = #searchExtT
 
    for ii = 1, #pathA do
-      local vv    = pathA[ii]
-      local mpath = vv.mpath
-      local fn    = pathJoin(vv.file, self:version())
-      found       = false
-      result      = nil
+      local vv       = pathA[ii]
+      local mpath    = vv.mpath
+      local versionT = vv.versionT
+      local version  = self:version() or 0
 
-      for i = 1, numExts do
-         local v        = searchExtT[i]
-         local f        = fn .. v
-         local attr     = lfs.attributes(f)
-         local readable = posix.access(f,"r")
-
-         if (readable and attr and attr.mode == "file") then
-            result = f
-            found  = true
-            break;
-         end
-      end
-
-      if (found) then
+      local fn       = versionT[version]
+      if (fn) then
+         result = fn
          local _, j = result:find(mpath, 1, true)
-         fullName  = result:sub(j+2):gsub("%.lua$","")
+         found      = true
+         fullName   = result:sub(j+2):gsub("%.lua$","")
          dbg.print{"fullName: ",fullName,"\n"}
-         dbg.print{"found:", found, " fn: ",fn,"\n"}
+         dbg.print{"found:    ",found, " fn: ",fn,"\n"}
          break
       end
    end
 
-
    if (found) then
       t.fn          = result
       t.modFullName = fullName
       t.modName     = sn
+      t.version     = extractVersion(t.modFullName, t.modName)
       dbg.print{"modName: ",sn," fn: ", result," modFullName: ", fullName,
                 " default: ",t.default,"\n"}
    else
@@ -457,117 +415,27 @@ end
 
 
 
---------------------------------------------------------------------------
--- This local function takes the file pointed to by the 
--- .version file and looks to see if that file exists
--- in the current mpath directory.  Note that this file
--- might have a .lua extension.
-
-local function followDotVersion(mpath, sn, version)
-   local accept_fn  = accept_fn
-   local fn         = pathJoin(mpath, sn, version)
-   local searchExtT = accept_extT()
-   local numExts    = #searchExtT
-   local result     = nil
-
-   for i = 1, numExts do
-      local v        = searchExtT[i]
-      local f        = fn .. v
-      local attr     = lfs.attributes(f)
-      local readable = posix.access(f,"r")
-
-      if (readable and attr and attr.mode == "file") then
-         result = f
-         break
-      end
-   end
-
-   return result
-end
-
-
-searchDefaultT = { "/default", "/.modulerc", "/.version" }
-
 --------------------------------------------------------------------------
 -- Look for the module name via a marked default.
 -- @param self A MName object
 -- @param pathA An array of paths to search
 -- @return True or false
 -- @return A table describing the module if found.
-function M.find_marked_default(self, pathA)
-   dbg.start{"MName:find_marked_default(pathA, t)"}
-   local usrName   = self:usrName()
-   local sn        = self:sn()
-   local accept_fn = accept_fn
-   local t         = module_locationT()
-   local found     = false
-   local result    = nil
-   local fullName  = ""
-   local modName   = ""
-   local Master    = Master
-
-   dbg.print{"usrName: ", usrName, "\n"}
-   dbg.print{"sn:      ", sn, "\n"}
-   if (sn ~= usrName) then
-      dbg.print{"Sn and user name do not match\n"}
-      return found, t
+function M.find_default(self, pathA, defaultEntry)
+   dbg.start{"MName:find_default(pathA, t)"}
+   local t        = module_locationT()
+   if (self:version()) then
+      return false, t
    end
-      
-   for ii = 1, #pathA do
-      local vv    = pathA[ii]
-      local mpath = vv.mpath
-      local fn    = vv.file
-      found       = false
-      result      = nil
-
-      for i = 1, 3 do
-         local v        = searchDefaultT[i]
-         local f        = fn .. v
-         local attr     = lfs.attributes(f)
-         local readable = posix.access(f,"r")
-
-         if (readable and attr and attr.mode == "file") then
-            result = f
-            if (v == "/default") then
-               result    = followDefault(result)
-               if (result) then
-                  t.default = 1
-                  found  = true
-                  break
-               end
-            elseif (v == "/.modulerc" or v == "/.version") then
-               local vf = versionFile(v, sn, result)
-               if (vf) then
-                  result = followDotVersion(mpath, sn, vf)
-                  if (result) then
-                     t.default = 1
-                     t.fn      = result
-                     dbg.print {"(1) .version: result: ", result,"\n"}
-                     found     = true
-                     break;
-                  end
-               end
-            end
-         end
-      end
-      if (found) then
-         dbg.print{"result: ",result,", mpath: ",mpath,"\n"}
-         local _, j = result:find(mpath, 1, true)
-         fullName  = result:sub(j+2):gsub("%.lua$","")
-         dbg.print{"fullName: ",fullName,", fn: ",fn,"\n"}
-         break
-      end
-   end
-
-   if (found) then
-      t.fn          = result
-      t.modFullName = fullName
-      t.modName     = self:sn()
-      dbg.print{"modName: ",sn," fn: ", result," modFullName: ", fullName,
-                " default: ",t.default,"\n"}
-   end
-
-   dbg.fini("MName:find_marked_default")
+   local found    = true
+   t.fn           = defaultEntry.fn
+   t.modFullName  = defaultEntry.fullName
+   t.modName      = self:sn()
+   t.default      = 1
+   t.version      = extractVersion(t.modFullName, t.modName)
+   dbg.print{"modName: ",t.modName," fn: ", t.fn," modFullName: ", t.modFullName,
+             " default: ",t.default," version: ",t.version,"\n"}
+   dbg.fini("MName:find_default")
    return found, t
 end
 
@@ -591,24 +459,21 @@ function M.find_latest(self, pathA)
       return found, t
    end
 
-
-   local result    = nil
    local fullName  = ""
    local modName   = ""
-   local Master    = Master
+   local result    = lastFileInPathA(pathA)
 
-
-   result          = lastFileInPathA(pathA)
    if (result) then
       local file    = result.file
       local _, j    = file:find(result.mpath, 1, true)
       t.modFullName = file:sub(j+2):gsub("%.lua$","")
       found         = true
-      t.default     = 1
+      t.default     = 0
       t.fn          = file
       t.modName     = sn
+      t.version     = extractVersion(t.modFullName, t.modName)
       dbg.print{"modName: ",sn," fn: ", file," modFullName: ", t.modFullName,
-                " default: ",t.default,"\n"}
+                " default: ",t.default," version: ",t.version,"\n"}
    end
 
    dbg.fini("MName:find_latest")
@@ -622,24 +487,30 @@ end
 -- @param pathA An array of paths to search
 -- @return True or false
 -- @return A table describing the module if found.
-function M.find_marked_default_between(self, pathA)
+function M.find_default_between(self, pathA, defaultEntry)
    dbg.start{"MName:find_marked_default_between(pathA, t)"}
 
    local t     = module_locationT()
    local found = false
 
-   found, t = self:find_marked_default(pathA, t)
+   found, t = self:find_default(pathA, defaultEntry)
 
    local left       = parseVersion(self._is)
    local right      = parseVersion(self._ie)
-   local version    = extractVersion(t.modFullName, t.modName)
+   local version    = t.version
    local pv         = parseVersion(version)
+
    if (pv < left or  pv > right) then
       found     = false
       t.default = 0
       t.fn      = nil
    end
 
+   dbg.print{"left:    ",left,"\n"}
+   dbg.print{"right:   ",right,"\n"}
+   dbg.print{"version: ",version,"\n"}
+   dbg.print{"pv:      ",pv,"\n"}
+   dbg.print{"default: ",t.default,"\n"}
    dbg.fini("MName:find_marked_default_between")
    return found, t
 end
@@ -658,11 +529,11 @@ function M.find_between(self, pathA)
    local found = false
    local a     = allVersions(pathA)
 
-   sort(a, function(a,b)
-             if (a.pv == b.pv) then
-                return a.idx < b.idx
+   sort(a, function(x,y)
+             if (x.pv == y.pv) then
+                return x.idx < y.idx
              else
-                return a.pv < b.pv
+                return x.pv  < y.pv
              end
            end
    )
@@ -686,6 +557,7 @@ function M.find_between(self, pathA)
       t.modFullName = v.file:sub(j+2):gsub("%.lua$","")
       t.default     = 0
       t.modName     = self:sn()
+      t.version     = extractVersion(t.modFullName, t.modName)
       found         = true
    end
 
@@ -712,10 +584,9 @@ function M.find(self)
    local fullName = ""
    local modName  = ""
    local sn       = self:sn()
-   local Master   = Master
    dbg.print{"MName:find sn: ",sn,"\n"}
 
-   local pathA = mt:locationTbl(sn)
+   local pathA, defaultEntry = mt:locationTbl(sn)
    if (pathA == nil or #pathA == 0) then
       dbg.print{"did not find key: \"",sn,"\" in mt:locationTbl()\n"}
       dbg.fini("MName:find")
@@ -726,7 +597,7 @@ function M.find(self)
    local stepA = self:steps()
    for i = 1, #stepA do
       local func = stepA[i]
-      found, t   = func(self, pathA)
+      found, t   = func(self, pathA, defaultEntry)
       dbg.print{"(1) t.fn: ", t.fn, "\n"}
       if (found) then
          break
