@@ -15,7 +15,7 @@
 --    a) setenv(...) is called in modfuncs
 --    b) This calls mcp:setenv(...)
 --    c) the mcp load version connects it to MasterControl:setenv
--- 
+--
 --  Suppose a user is requesting to unload a module which contains a
 --  setenv command.
 --    0. An MCP unload object is constructed.
@@ -24,6 +24,20 @@
 --    3. The unload MCP objects maps this to MasterControl:unsetenv
 --    4. The users' environment variable is unset.
 --
+--  As a convention: MCP is always for loads while the purpose of mcp is
+--  dynamic.
+--
+--  The rational behind this design is to support all the ways a modulefile can
+--  be evaluated.
+--
+--  In a module file, the change to the environment upon loading are specified:
+--  set a variable, prepend something to a variable, etc. When you 'unload' the
+--  module, these changes need to get reversed. So depending on the 'mode' (load,
+--  unloading, ...), 'setenv' will have a different meaning. Instead of using 'if'
+--  statements, the current design allows in an elegant way to the define the
+--  different 'setenv' commands. There are at least 8 modes and they can be
+--  found in the function 'M.build' below.
+--
 -- @classmod MasterControl
 
 require("strict")
@@ -66,6 +80,7 @@ require("TermWidth")
 require("string_utils")
 require("inherits")
 require("utils")
+require("myGlobals")
 
 local M            = {}
 local BeautifulTbl = require("BeautifulTbl")
@@ -82,6 +97,7 @@ local concatTbl    = table.concat
 local decode64     = base64.decode64
 local encode64     = base64.encode64
 local getenv       = os.getenv
+local hook         = require("Hook")
 local remove       = table.remove
 local pack         = (_VERSION == "Lua 5.1") and argsPack or table.pack
 local Exit         = os.exit
@@ -98,7 +114,10 @@ function M.mustLoad()
    if (#aa > 0) then
       local luaprog = "@path_to_lua@/lua"
       if (luaprog:sub(1,1) == "@") then
-         luaprog = findInPath("lua")
+         luaprog = find_exec_path("lua")
+         if (luaprog == nil) then
+            LmodError("Unable to find the lua program")
+         end
       end
       local cmdA = {}
       cmdA[#cmdA+1] = luaprog
@@ -132,14 +151,14 @@ function M.mustLoad()
       local a = {}
 
       if (#uA > 0) then
-         a[#a+1] = "\nThe following module(s) are unknown: "
+         a[#a+1] = "The following module(s) are unknown: "
          a[#a+1] = concatTbl(uA, " ")
-         a[#a+1] = "\n\n   Please check the spelling or version number. "
+         a[#a+1] = "\n\nPlease check the spelling or version number. "
          a[#a+1] = "Also try \"module spider ...\"\n"
       end
 
       if (#kA > 0) then
-         a[#a+1] = "\nThese module(s) exist but cannot be loaded as requested: "
+         a[#a+1] = "These module(s) exist but cannot be loaded as requested: "
          a[#a+1] = concatTbl(kA,", ")
          a[#a+1] = "\n\n   Try: \"module spider "
          a[#a+1] = concatTbl(kB, " ")
@@ -165,7 +184,7 @@ local function mAList(mA)
 end
 
 --------------------------------------------------------------------------
--- Return the name of the derived MC object. 
+-- Return the name of the derived MC object.
 -- @param self A MasterControl object
 function M.name(self)
    return self.my_name
@@ -199,30 +218,35 @@ end
 -- @param name the name of the derived object.
 -- @param[opt] mode An optional mode for building the *access* object.
 -- @return A derived MasterControl Object.
+local s_nameTbl     = false
 function M.build(name,mode)
 
-   local nameTbl          = {}
-   local MCLoad           = require('MC_Load')
-   local MCUnload         = require('MC_Unload')
-   local MCMgrLoad        = require('MC_MgrLoad')
-   local MCRefresh        = require('MC_Refresh')
-   local MCShow           = require('MC_Show')
-   local MCAccess         = require('MC_Access')
-   local MCSpider         = require('MC_Spider')
-   local MCComputeHash    = require('MC_ComputeHash')
-   nameTbl["load"]        = MCLoad
-   nameTbl["mgrload"]     = MCMgrLoad
-   nameTbl["unload"]      = MCUnload
-   nameTbl["refresh"]     = MCRefresh
-   nameTbl["show"]        = MCShow
-   nameTbl["access"]      = MCAccess
-   nameTbl["spider"]      = MCSpider
-   nameTbl["computeHash"] = MCComputeHash
-   nameTbl.default        = MCLoad
+   if (not s_nameTbl) then
+      local MCLoad        = require('MC_Load')
+      local MCUnload      = require('MC_Unload')
+      local MCMgrLoad     = require('MC_MgrLoad')
+      local MCRefresh     = require('MC_Refresh')
+      local MCShow        = require('MC_Show')
+      local MCAccess      = require('MC_Access')
+      local MCSpider      = require('MC_Spider')
+      local MCComputeHash = require('MC_ComputeHash')
+      s_nameTbl = {
+         ["load"]        = MCLoad,        -- Normal loading of modules
+         ["mgrload"]     = MCMgrLoad,     -- for collections (loads in modules are ignored)
+         ["unload"]      = MCUnload,      -- Unload modules
+         ["refresh"]     = MCRefresh,     -- for subshells, sets the aliases again
+         ["computeHash"] = MCComputeHash, -- Generate a hash value for the contents of the module
+         ["refresh"]     = MCRefresh,     -- for subshells, sets the aliases again
+         ["show"]        = MCShow,        -- show the module function instead.
+         ["access"]      = MCAccess,      -- for whatis, help
+         ["spider"]      = MCSpider,      -- Process module files for spider operations
+      }
+      s_nameTbl.default        = MCLoad
+   end
 
-   local o                = valid_name(nameTbl, name):create()
+   local o                = valid_name(s_nameTbl, name):create()
    o:_setMode(mode or name)
-   
+
 
    dbg.print{"Setting mcp to ", o:name(),"\n"}
    return o
@@ -247,8 +271,10 @@ end
 -- @param self A MasterControl object
 -- @param mA A array of MName objects.
 -- @return An array of statuses
+
+s_adminT = {}
+
 function M.load(self, mA)
-   LMOD_IGNORE_CACHE = true
    local master = Master:master()
 
    if (dbg.active()) then
@@ -258,55 +284,64 @@ function M.load(self, mA)
 
    local a = master.load(mA)
    if (not quiet()) then
-
-      local mt      = MT:mt()
-      local t       = {}
-      readAdmin()
-      for i = 1, #mA do
-         local mname      = mA[i]
-         local sn         = mname:sn()
-         if (mt:have(sn,"active")) then
-            local moduleFn  = mt:fileName(sn)
-            local modFullNm = mt:fullName(sn)
-            local message
-            local key
-            if (adminT[moduleFn]) then
-               message = adminT[moduleFn]
-               key     = moduleFn
-            elseif (adminT[modFullNm]) then
-               message = adminT[modFullNm]
-               key     = modFullNm
-            end
-
-            if (message) then
-               t[key] = message
-            end
-         end
-      end
-
-      if (next(t)) then
-         local term_width  = TermWidth()
-         if (term_width < 40) then
-            term_width = 80
-         end
-         local bt
-         local a       = {}
-         local border  = string.rep("-", term_width-1)
-         io.stderr:write("\n",border,"\n","Module(s):\n",border,"\n")
-         for k, v in pairs(t) do
-            io.stderr:write("\n",k," :\n")
-            a[1] = { " ", v}
-            bt = BeautifulTbl:new{tbl=a, wrapped=true, column=term_width-1}
-            io.stderr:write(bt:build_tbl(), "\n")
-         end
-         io.stderr:write(border,"\n\n")
-      end
+      self:registerAdminMsg(mA)
    end
 
    dbg.fini("MasterControl:load")
+
    return a
 end
 
+function M.registerAdminMsg(self, mA)
+   local mt      = MT:mt()
+   local t       = s_adminT
+   readAdmin()
+   for i = 1, #mA do
+      local mname      = mA[i]
+      local sn         = mname:sn()
+      if (mt:have(sn,"active")) then
+         local moduleFn  = mt:fileName(sn)
+         local modFullNm = mt:fullName(sn)
+         local message
+         local key
+         if (adminT[moduleFn]) then
+            message = adminT[moduleFn]
+            key     = moduleFn
+         elseif (adminT[modFullNm]) then
+            message = adminT[modFullNm]
+            key     = modFullNm
+         end
+
+         if (message) then
+            t[key] = message
+         end
+      end
+   end
+end
+
+
+-------------------------------------------------------------------
+-- Output any admin message collected from loading.
+function M.reportAdminMsgs()
+   local t = s_adminT
+   if (next(t) ) then
+      local term_width  = TermWidth()
+      local bt
+      local a       = {}
+      local border  = colorize("red",string.rep("-", term_width-1))
+      io.stderr:write("\n",border,"\n",
+                      "There are messages associated with the following module(s):\n",
+                      border,"\n")
+      for k, v in pairsByKeys(t) do
+         io.stderr:write("\n",k,":\n")
+         a[1] = { " ", v}
+         bt = BeautifulTbl:new{tbl=a, wrapped=true, column=term_width-1}
+         io.stderr:write(bt:build_tbl(), "\n")
+      end
+      io.stderr:write(border,"\n\n")
+   end
+end
+
 -------------------------------------------------------------------
 -- Load a list of module but ignore any warnings.
 -- @param self A MasterControl object
@@ -388,7 +423,7 @@ function M.prepend_path(self, t)
    local sep      = t.delim or ":"
    local name     = t[1]
    local value    = t[2]
-   local nodups   = t.nodups
+   local nodups   = not allow_dups( not t.nodups)
    local priority = (-1)*(t.priority or 0)
    dbg.print{"name:\"",name,"\", value: \"",value,
              "\", delim=\"",sep,"\", nodups=\"",nodups,
@@ -451,7 +486,7 @@ function M.remove_path(self, t)
    if (varTbl[name] == nil) then
       varTbl[name] = Var:new(name,nil, sep)
    end
-   varTbl[name]:remove(tostring(value), where, priority)
+   varTbl[name]:remove(tostring(value), where, priority, nodups)
    dbg.fini("MasterControl:remove_path")
 end
 
@@ -486,9 +521,14 @@ end
 -- @param value the environment variable value.
 -- @param respect If true, then respect the old value.
 function M.setenv(self, name, value, respect)
+   name = name:trim()
    dbg.start{"MasterControl:setenv(\"",name,"\", \"",value,"\", \"",
               respect,"\")"}
 
+   if (value == nil) then
+      LmodError("setenv(\"",name,"\") is not valid, a value is required")
+   end
+
    if (respect and getenv(name)) then
       dbg.print{"Respecting old value"}
       dbg.fini("MasterControl:setenv")
@@ -510,6 +550,7 @@ end
 -- @param value the environment variable value.
 -- @param respect If true, then respect the old value.
 function M.unsetenv(self, name, value, respect)
+   name = name:trim()
    dbg.start{"MasterControl:unsetenv(\"",name,"\", \"",value,"\")"}
 
    if (respect and getenv(name) ~= value) then
@@ -535,6 +576,7 @@ end
 -- @param name the environment variable name.
 -- @param value the environment variable value.
 function M.pushenv(self, name, value)
+   name = name:trim()
    dbg.start{"MasterControl:pushenv(\"",name,"\", \"",value,"\")"}
 
    ----------------------------------------------------------------
@@ -542,6 +584,10 @@ function M.pushenv(self, name, value)
    -- doesn't exist then use the name's value as the initial value
    -- for "stackName".
 
+   if (value == nil) then
+      LmodError("pushenv(\"",name,"\") is not valid, a value is required")
+   end
+
    local stackName = "__LMOD_STACK_" .. name
    local v64       = nil
    local v         = getenv(name)
@@ -577,12 +623,10 @@ end
 -- @param name the environment variable name.
 -- @param value the environment variable value.
 function M.popenv(self, name, value)
+   name = name:trim()
    dbg.start{"MasterControl:popenv(\"",name,"\", \"",value,"\")"}
 
    local stackName = "__LMOD_STACK_" .. name
-
-   local v = nil
-
    if (varTbl[stackName] == nil) then
       varTbl[stackName] = Var:new(stackName)
    end
@@ -613,6 +657,7 @@ end
 -- @param name the environment variable name.
 -- @param value the environment variable value.
 function M.set_alias(self, name, value)
+   name = name:trim()
    dbg.start{"MasterControl:set_alias(\"",name,"\", \"",value,"\")"}
 
 
@@ -624,17 +669,18 @@ function M.set_alias(self, name, value)
 end
 
 --------------------------------------------------------------------------
--- Unset a shell alias. 
+-- Unset a shell alias.
 -- @param self A MasterControl Object.
 -- @param name the environment variable name.
 -- @param value the environment variable value.
 function M.unset_alias(self, name, value)
+   name = name:trim()
    dbg.start{"MasterControl:unset_alias(\"",name,"\", \"",value,"\")"}
 
    if (varTbl[name] == nil) then
       varTbl[name] = Var:new(name)
    end
-   varTbl[name]:unsetAlias(value)
+   varTbl[name]:unsetAlias()
    dbg.fini("MasterControl:unset_alias")
 end
 
@@ -645,6 +691,7 @@ end
 -- @param name the environment variable name.
 -- @param value the environment variable value.
 function M.set_shell_function(self, name, bash_function, csh_function)
+   name = name:trim()
    dbg.start{"MasterControl:set_shell_function(\"",name,"\", \"",bash_function,"\"",
              "\", \"",csh_function,"\""}
 
@@ -662,6 +709,7 @@ end
 -- @param name the environment variable name.
 -- @param value the environment variable value.
 function M.unset_shell_function(self, name, bash_function, csh_function)
+   name = name:trim()
    dbg.start{"MasterControl:unset_shell_function(\"",name,"\", \"",bash_function,"\"",
              "\", \"",csh_function,"\""}
 
@@ -686,7 +734,7 @@ function M.add_property(self, name, value)
    local mFull   = mStack:fullName()
    local mt      = MT:mt()
    local mname   = MName:new("load",mFull)
-   mt:add_property(mname:sn(), name, value)
+   mt:add_property(mname:sn(), name:trim(), value)
 end
 
 --------------------------------------------------------------------------
@@ -695,32 +743,39 @@ end
 -- @param name A property name
 -- @param value A property value.
 function M.remove_property(self, name, value)
+   name = name:trim()
    local mStack  = ModuleStack:moduleStack()
    local mFull   = mStack:fullName()
    local mt      = MT:mt()
    local mname   = MName:new("mt",mFull)
-   mt:remove_property(mname:sn(), name, value)
+   mt:remove_property(mname:sn(), name:trim(), value)
 end
 
 
 --------------------------------------------------------------------------
 -- Report the modulefiles stack for error report.
-local function moduleStackTraceBack()
+function moduleStackTraceBack(msg)
    local mStack = ModuleStack:moduleStack()
-   if (mStack:empty()) then return end
+   msg = msg or "While processing the following module(s):\n"
+   if (mStack:empty()) then return "" end
 
    local aa = {}
-   aa[1] = { "Module fullname", "Module Filename"}
-   aa[2] = { "---------------", "---------------"}
+   aa[1]    = { "  ", "Module fullname", "Module Filename"}
+   aa[2]    = { "  ", "---------------", "---------------"}
 
-   while (not mStack:empty()) do
-      aa[#aa+1] = {mStack:fullName(), mStack:fileName()}
-      mStack:pop()
+   local a  = mStack:traceBack()
+
+   for i = 1,#a do
+      local entry = a[i]
+      aa[#aa+1] = {"  ",entry.fullName, entry.fn}
    end
 
    local bt = BeautifulTbl:new{tbl=aa}
-   io.stderr:write("\n","While processing the following module(s):\n\n",
-                   bt:build_tbl(),"\n")
+
+   local bb = {}
+   bb[#bb+1] = msg
+   bb[#bb+1] = bt:build_tbl()
+   return concatTbl(bb,"")
 end
 
 --------------------------------------------------------------------------
@@ -733,13 +788,28 @@ end
 --------------------------------------------------------------------------
 -- Print msgs, traceback then exit.
 function LmodSystemError(...)
-   io.stderr:write("\n", colorize("red", "Lmod has detected the following error: "))
-   local arg = pack(...)
-   for i = 1, arg.n do
-      io.stderr:write(tostring(arg[i]))
+   local label  = colorize("red", "Lmod has detected the following error: ")
+   local twidth = TermWidth()
+   local s      = {}
+   s[#s+1] = buildMsg(twidth, label, ...)
+   s[#s+1] = "\n"
+
+   local a = concatTbl(stackTraceBackA,"")
+   if (a:len() > 0) then
+       s[#s+1] = a
+       s[#s+1] = "\n"
    end
-   io.stderr:write("\n")
-   moduleStackTraceBack()
+
+   a = moduleStackTraceBack()
+   if (a ~= "") then
+       s[#s+1] = a
+       s[#s+1] = "\n"
+   end
+
+   s = hook.apply("msgHook","lmoderror",s)
+   s = concatTbl(s,"")
+
+   io.stderr:write(s,"\n")
    LmodErrorExit()
 end
 
@@ -756,13 +826,22 @@ end
 -- @param self A MasterControl object.
 function M.warning(self, ...)
    if (not quiet() and  haveWarnings()) then
-      io.stderr:write("\n",colorize("red", "Lmod Warning: "))
-      local arg = pack(...)
-      for i = 1, arg.n do
-         io.stderr:write(tostring(arg[i]))
+      local label  = colorize("red", "Lmod Warning: ")
+      local twidth = TermWidth()
+      local s      = {}
+      s[#s+1] = buildMsg(twidth, label, ...)
+      s[#s+1] = "\n"
+
+      local a = moduleStackTraceBack()
+      if (a ~= "") then
+          s[#s+1] = a
+          s[#s+1] = "\n"
       end
-      io.stderr:write("\n")
-      moduleStackTraceBack()
+
+      s = hook.apply("msgHook","lmodwarning",s)
+      s = concatTbl(s,"")
+
+      io.stderr:write(s,"\n")
       setWarningFlag()
    end
 end
@@ -790,7 +869,6 @@ end
 -- @param mA An array of MNname objects.
 function M.prereq(self, mA)
    local mt        = MT:mt()
-   local a         = {}
    local mStack    = ModuleStack:moduleStack()
    local mFull     = mStack:fullName()
    local masterTbl = masterTbl()
@@ -813,8 +891,8 @@ function M.prereq(self, mA)
 
    dbg.print{"number found: ",#a,"\n"}
    if (#a > 0) then
-      local s = concatTbl(a,", ")
-      LmodError("Cannot load module \"",mFull,"\" without these modules loaded:\n  ",
+      local s = concatTbl(a," ")
+      LmodError("Cannot load module \"",mFull,"\" without these module(s) loaded:\n  ",
             s,"\n")
    end
    dbg.fini("MasterControl:prereq")
@@ -829,7 +907,6 @@ function M.conflict(self, mA)
 
 
    local mt        = MT:mt()
-   local a         = {}
    local mStack    = ModuleStack:moduleStack()
    local mFull     = mStack:fullName()
    local masterTbl = masterTbl()
@@ -842,23 +919,32 @@ function M.conflict(self, mA)
 
    local a = {}
    for i = 1, #mA do
-      local mname   = mA[i]
-      local v       = mname:usrName()
-      local sn      = mname:sn()
-      local version = extractVersion(v, sn)
-      local found   = false
-      if (version) then
-         found = mt:fullName(sn) == mname:usrName()
-      else
-         found = mt:have(sn,"active")
-      end
-      if (found) then
-         a[#a+1] = v
+      local mname = mA[i]
+      local sn    = mname:sn()
+      if (mt:have(sn,"active")) then
+         a[#a+1]  = mname:usrName()
       end
    end
+
+   --for i = 1, #mA do
+   --   local mname   = mA[i]
+   --   local v       = mname:usrName()
+   --   local sn      = mname:sn()
+   --   local version = extractVersion(v, sn)
+   --   local found   = false
+   --   if (version) then
+   --      found = mt:fullName(sn) == mname:usrName()
+   --   else
+   --      found = mt:have(sn,"active")
+   --   end
+   --   if (found) then
+   --      a[#a+1] = v
+   --   end
+   --end
+
    if (#a > 0) then
       local s = concatTbl(a," ")
-      LmodError("Cannot load module \"",mFull,"\" because these modules are loaded:\n  ",
+      LmodError("Cannot load module \"",mFull,"\" because these module(s) are loaded:\n  ",
             s,"\n")
    end
    dbg.fini("MasterControl:conflict")
@@ -871,7 +957,6 @@ end
 -- @param mA An array of MNname objects.
 function M.prereq_any(self, mA)
    local mt        = MT:mt()
-   local a         = {}
    local mStack    = ModuleStack:moduleStack()
    local mFull     = mStack:fullName()
    local masterTbl = masterTbl()
@@ -901,7 +986,7 @@ function M.prereq_any(self, mA)
 
    if (not found) then
       local s = concatTbl(a," ")
-      LmodError("Cannot load module \"",mFull,"\".  At least one of these modules must be loaded:\n  ",
+      LmodError("Cannot load module \"",mFull,"\".  At least one of these module(s) must be loaded:\n  ",
             concatTbl(a,", "),"\n")
    end
    dbg.fini("MasterControl:prereq_any")
@@ -947,7 +1032,7 @@ function M.familyLoaded()
          bb[#bb+1] = usrName
       end
    end
-         
+
    dbg.fini("MC:familyLoaded")
    return aa, bb
 end
@@ -1026,36 +1111,61 @@ function M.family(self, name)
    dbg.fini("MasterControl:family")
 end
 
+--------------------------------------------------------------------------
+-- Return the user's shell
+-- @param self A MasterControl object
 function M.myShellName(self)
    local master = _G.Master:master()
    return master.shell:name()
 end
 
+--------------------------------------------------------------------------
+-- Return the current file name.
+-- @param self A MasterControl object
 function M.myFileName(self)
    local mStack = ModuleStack:moduleStack()
    return mStack:fileName()
 end
 
+--------------------------------------------------------------------------
+-- Return the full name of the current module.  Typically name/version.
+-- @param self A MasterControl object.
 function M.myModuleFullName(self)
    local mStack = ModuleStack:moduleStack()
    return mStack:fullName()
 end
 
+--------------------------------------------------------------------------
+-- Return the user name of the current module.  This is the name the user
+-- specified.  It could a full name (name/version) or just the name.
+-- @param self A MasterControl object.
 function M.myModuleUsrName(self)
    local mStack = ModuleStack:moduleStack()
    return mStack:usrName()
 end
 
+--------------------------------------------------------------------------
+-- Return the name of the modules.  That is the name of the module w/o a
+-- version.
+-- @param self A MasterControl object
 function M.myModuleName(self)
    local mStack = ModuleStack:moduleStack()
    return mStack:sn()
 end
 
+--------------------------------------------------------------------------
+-- Return the version if any.  If there is no version, for example a meta
+-- module then the version is "".
+-- @param self A MasterControl object
 function M.myModuleVersion(self)
    local mStack = ModuleStack:moduleStack()
    return mStack:version()
 end
 
+--------------------------------------------------------------------------
+-- Unset the family name.
+-- @param self A MasterControl object
+-- @param name A family name.
 function M.unset_family(self, name)
    local mt     = MT:mt()
 
@@ -1065,6 +1175,10 @@ function M.unset_family(self, name)
    dbg.fini("MasterControl:unset_family")
 end
 
+--------------------------------------------------------------------------
+-- Perform a user requested inheritance.  Note that this function remains
+-- the same depending on if it is a load or unload.
+-- @param self A MasterControl object
 function M.inherit(self)
    local master = Master:master()
    dbg.start{"MasterControl:inherit()"}
@@ -1073,18 +1187,28 @@ function M.inherit(self)
    dbg.fini("MasterControl:inherit")
 end
 
+
+--------------------------------------------------------------------------
+-- Return True when in spider mode.  This version is always false.
+-- @param self A MasterControl object
 function M.is_spider(self)
    dbg.start{"MasterControl:is_spider()"}
    dbg.fini("MasterControl:is_spider")
    return false
 end
 
+--------------------------------------------------------------------------
+-- Set the type (or mode) of the current MasterControl object.
+-- @param self A MasterControl object
 function M._setMode(self, mode)
    dbg.start{"MasterControl:_setMode(\"",mode,"\")"}
    self._mode = mode
    dbg.fini("MasterControl:_setMode")
 end
 
+--------------------------------------------------------------------------
+-- Return the type (or mode) of the current MasterControl object.
+-- @param self A MasterControl object
 function M.mode(self)
    dbg.start{"MasterControl:mode()"}
    dbg.print{"mode: ", self._mode,"\n"}
@@ -1093,9 +1217,9 @@ function M.mode(self)
 end
 
 --------------------------------------------------------------------------
--- MasterControl:execute() - place a string that will be executed when
---                           the output from Lmod eval'ed.  
-
+-- Place a string that will be executed when the output from Lmod eval'ed.
+-- @param self A MasterControl object
+-- @param t A table containing A mode array and a command.
 function M.execute(self, t)
    dbg.start{"MasterControl:execute(t)"}
    local a      = t.modeA or {}
@@ -1111,14 +1235,9 @@ function M.execute(self, t)
    dbg.fini("MasterControl:execute")
 end
 
-
-
-
--------------------------------------------------------------------
--- Quiet Functions
--------------------------------------------------------------------
-
-
+--------------------------------------------------------------------------
+-- The quiet function.
+-- @param self A MasterControl object
 function M.quiet(self, ...)
    -- very Quiet !!!
 end
