@@ -1,13 +1,13 @@
 --module('Version')
 local M={}
-function M.tag()  return "5.8"   end
+function M.tag()  return "6.6"   end
 function M.git()
-   local s = "(5.8)"
+   local s = "(6.6)"
    if (s == "@" .. "git@")     then s = "" end
    if (s == "("..M.tag()..")") then s = "" end
    return s
 end
-function M.date() return "2014-11-04 12:46" end
+function M.date() return "2016-10-13 13:28 -05:00" end
 function M.name()
   local a = {}
   a[#a+1] = M.tag()
