@@ -47,17 +47,40 @@
 --  THE SOFTWARE.
 --
 --------------------------------------------------------------------------
-
-
-local cmd = arg[0]
-
-local i,j = cmd:find(".*/")
-local cmd_dir = "./"
-if (i) then
-   cmd_dir = cmd:sub(1,j)
+local sys_lua_path = "@sys_lua_path@"
+if (sys_lua_path:sub(1,1) == "@") then
+   sys_lua_path = package.path
 end
-package.path = cmd_dir .. "../tools/?.lua;" ..
-               cmd_dir .. "?.lua;"       .. package.path
+
+local sys_lua_cpath = "@sys_lua_cpath@"
+if (sys_lua_cpath:sub(1,1) == "@") then
+   sys_lua_cpath = package.cpath
+end
+
+package.path   = sys_lua_path
+package.cpath  = sys_lua_cpath
+
+local arg_0    = arg[0]
+local posix    = require("posix")
+local readlink = posix.readlink
+local stat     = posix.stat
+
+local st       = stat(arg_0)
+while (st.type == "link") do
+   arg_0 = readlink(arg_0)
+   st    = stat(arg_0)
+end
+
+local ia,ja = arg_0:find(".*/")
+local cmd_dir = "./"
+if (ia) then
+   cmd_dir = arg_0:sub(1,ja)
+end
+
+package.path  = cmd_dir .. "../tools/?.lua;" ..
+                cmd_dir .. "?.lua;"          ..
+                sys_lua_path
+package.cpath = sys_lua_cpath
 
 require("strict")
 require("string_utils")
@@ -76,11 +99,8 @@ end
 
 function isDir(d)
    if (d == nil) then return false end
-
    local attr    = lfs.attributes(d)
-   local results = (attr and attr.mode == "directory")
-
-   return result
+   return (attr and attr.mode == "directory")
 end
 
 function myInsert(appendFlg, existFlg)
@@ -108,7 +128,7 @@ function myClean(cleanFlg)
                    path = path:sub(1,-2)
                 end
                 if (path == "") then path = false end
-                return path 
+                return path
              end
    else
       return function (path)
@@ -150,10 +170,10 @@ function main()
    ------------------------------------------------------------------------
    -- Convert empty string input values into false and clean path if requested
    local valueA    = {}
-   for i = 1,#pargs do
-      valueA[i] = cleanPath(pargs[i])
+   for j = 1,#pargs do
+      valueA[j] = cleanPath(pargs[j])
    end
-   
+
 
    ------------------------------------------------------------------------
    -- Convert empty string envVar values into false and clean path if requested
@@ -166,15 +186,15 @@ function main()
    ------------------------------------------------------------------------
    -- Make a hash table of input values
    local valueT = {}
-   for i = 1, #valueA do
-      valueT[valueA[i]] = true
+   for j = 1, #valueA do
+      valueT[valueA[j]] = true
    end
 
    ------------------------------------------------------------------------
    -- Remove any entries in input from envVarA
    local newA = {}
-   for i = 1, #envVarA do
-      local v = envVarA[i]
+   for j = 1, #envVarA do
+      local v = envVarA[j]
       if (not valueT[v]) then
          if (v == false) then v = "" end
          newA[#newA+1] = v
@@ -183,9 +203,9 @@ function main()
 
    ------------------------------------------------------------------------
    -- Insert/append new entries with magic insert function.
-   
-   for i = 1, #valueA do
-      local v = valueA[i]
+
+   for j = 1, #valueA do
+      local v = valueA[j]
       if (v == false) then v = "" end
       insert(newA, v)
    end
