@@ -1,5 +1,5 @@
 --------------------------------------------------------------------------
--- The all the user commands are implemented here.
+-- The all the user sub-commands are implemented here.
 -- @module cmdfuncs
 
 require("strict")
@@ -54,14 +54,14 @@ local hook         = require("Hook")
 local lfs          = require("lfs")
 local posix        = require("posix")
 local pack         = (_VERSION == "Lua 5.1") and argsPack or table.pack
-local unpack       = unpack or table.unpack
+local unpack       = (_VERSION == "Lua 5.1") and unpack or table.unpack
 
 --------------------------------------------------------------------------
--- Access(): Both Help and Whatis functions funnel their actions through
+-- Both Help and Whatis functions funnel their actions through
 -- the Access function. MC_Access defines real functions for both M.help
 -- and M.access.  The mcp.accessMode function activates one or the other
 -- depending on what mode Access is called with.
-
+-- @param mode Whether this function has be called via *Help* or *Whatis*.
 local function Access(mode, ...)
    local master    = Master:master()
    local shell     = master.shell
@@ -83,11 +83,11 @@ local function Access(mode, ...)
 end
 
 --------------------------------------------------------------------------
--- findNamedCollections:  This helper function walks the ~/.lmod.d
--- directory and reports back the list of named collections.
--- Note that names that start with "." or end with "~" or start with
--- "__" are ignored.
-
+-- This helper function walks the ~/.lmod.d directory and reports back
+-- the list of named collections. Note that names that start with "."
+-- or end with "~" or start with "__" are ignored.
+-- @param a An array containing the results.
+-- @param path The Lmod.d directory path.
 local function findNamedCollections(a,path)
    if (not isDir(path)) then return end
    for file in lfs.dir(path) do
@@ -98,16 +98,24 @@ local function findNamedCollections(a,path)
          if (attr and attr.mode == "directory") then
             findNamedCollections(a,f)
          else
-            a[#a+1] = f
+            local idx    = file:find("%.")
+            local accept = (not idx) and (not LMOD_SYSTEM_NAME)
+            if (idx and LMOD_SYSTEM_NAME) then
+               accept    = file:sub(idx+1,-1) == LMOD_SYSTEM_NAME
+               f         = pathJoin(path, file:sub(1,idx-1))
+            end
+            if (accept) then
+               a[#a+1] = f
+            end
          end
       end
    end
+   table.sort(a)
 end
 
 ------------------------------------------------------------------------
--- Avail(): just convert the vararg into an actual array and call
---          master.avail to do the real work.
-
+-- Just convert the vararg into an actual array and call
+-- master.avail to do the real work.
 function Avail(...)
    local master = Master:master()
    local arg    = pack(...)
@@ -115,27 +123,59 @@ function Avail(...)
 end
 
 --------------------------------------------------------------------------
--- GetDefault(): get the command line argument and use MT:getMTfromFile()
---               to read the module table from the file and use that
---               collections of module to load.  This routine is deprecated
---               and will be removed.  Use restore instead.
+-- Report the modules in the requested collection
+function CollectionLst(collection)
+   collection  = collection or "default"
+   dbg.start{"CollectionLst(",collection,")"}
+   local masterTbl = masterTbl()
+   local sname     = (LMOD_SYSTEM_NAME == nil) and "" or "." .. LMOD_SYSTEM_NAME
+   local path      = pathJoin(os.getenv("HOME"), ".lmod.d", collection .. sname)
+   local mt        = MT:mt()
+   local a         = mt:reportContents{fn=path, name=collection}
+   local shell     = Master:master().shell
+   local cwidth    = masterTbl.rt and LMOD_COLUMN_TABLE_WIDTH or TermWidth()
+   if (masterTbl.terse) then
+      for i = 1,#a do
+         shell:echo(a[i].."\n")
+      end
+   else
+      if (#a < 1) then
+         LmodWarning("No collection named \"",collection,"\" found.")
+         dbg.fini("CollectionLst")
+         return
+      end
+      shell:echo("Collection \"",collection,"\" contains: \n")
+      local b = {}
+      for i = 1,#a do
+         b[#b+1] = { "   " .. i .. ")", a[i] }
+      end
+      local ct = ColumnTable:new{tbl=b, gap = 0, width = cwidth}
+      shell:echo(ct:build_tbl(),"\n")
+   end
+   dbg.fini("CollectionLst")
+end
 
 
-function GetDefault(a)
-   a          = a or "default"
-   dbg.start{"GetDefault(",a,")"}
+--------------------------------------------------------------------------
+-- Get the command line argument and use MT:getMTfromFile()
+-- to read the module table from the file and use that
+-- collections of module to load.  This routine is deprecated
+-- and will be removed.  Use restore instead.
+-- @param collection The collection name (default="default")
+function GetDefault(collection)
+   collection  = collection or "default"
+   dbg.start{"GetDefault(",collection,")"}
 
-   local sname = (LMOD_SYSTEM_NAME == nil) and "" or "." .. LMOD_SYSTEM_NAME 
-   local path = pathJoin(os.getenv("HOME"), ".lmod.d", a .. sname)
-   local mt   = MT:mt()
-   mt:getMTfromFile{fn=path, name=a}
+   local sname = (LMOD_SYSTEM_NAME == nil) and "" or "." .. LMOD_SYSTEM_NAME
+   local path  = pathJoin(os.getenv("HOME"), ".lmod.d", collection .. sname)
+   local mt    = MT:mt()
+   mt:getMTfromFile{fn=path, name=collection, }
    dbg.fini("GetDefault")
 end
 
 --------------------------------------------------------------------------
--- Help():  Define the prtHdr function and use the helper function Access()
---          to report the Help message to the user.
-
+-- Define the prtHdr function and use the helper function Access()
+-- to report the Help message to the user.
 function Help(...)
 
    prtHdr = function()
@@ -153,16 +193,14 @@ function Help(...)
 end
 
 --------------------------------------------------------------------------
--- Keyword(): Use the list of user requested keywords to be searched
---            through the spider cache.
-
-
+-- Use the list of user requested keywords to be searched
+-- through the spider cache.
 function Keyword(...)
    dbg.start{"Keyword(",concatTbl({...},","),")"}
 
    local master  = Master:master()
    local shell   = master.shell
-   local cache   = Cache:cache()
+   local cache   = Cache:cache{buildCache=true}
    local moduleT = cache:build()
    local s
    local dbT = {}
@@ -182,24 +220,23 @@ function Keyword(...)
    ia = ia+1; a[ia] = "\n"
 
    spider:Level0Helper(dbT,a)
-   
+
    shell:echo(concatTbl(a,""))
 
    dbg.fini("Keyword")
 end
 
 --------------------------------------------------------------------------
--- List(): List the loaded modulefile
-
-
+-- List the loaded modulefile
 function List(...)
    dbg.start{"List(...)"}
    local masterTbl = masterTbl()
    local shell     = Master:master().shell
-   local mt = MT:mt()
-   local totalA = mt:list("userName","any")
+   local mt        = MT:mt()
+   local totalA    = mt:list("userName","any")
+   local cwidth    = masterTbl.rt and LMOD_COLUMN_TABLE_WIDTH or TermWidth()
    if (#totalA < 1) then
-      LmodMessage("No modules loaded\n")
+      shell:echo("No modules loaded\n")
       dbg.fini("List")
       return
    end
@@ -232,7 +269,7 @@ function List(...)
          for j = 1, wanted.n do
             local p = wanted[j]
             if (full:find(p)) then
-               io.stderr:write(full,"\n")
+               shell:echo(full.."\n")
             end
          end
       end
@@ -251,6 +288,7 @@ function List(...)
       local mname = MName:new("mt",activeA[i])
       local sn    = mname:sn()
       local full  = mt:fullName(sn)
+      dbg.print{"activeA[i]: ",activeA[i],", sn: ",sn,", full: ",full,"\n"}
       for j = 1, wanted.n do
          local p = wanted[j]
          if (full:find(p)) then
@@ -263,9 +301,11 @@ function List(...)
    if (kk == 0) then
       b[#b+1] = "  None found.\n"
    else
-      local ct = ColumnTable:new{tbl=a, gap=0, len=length}
-      b[#b+1] = ct:build_tbl()
-      b[#b+1] = "\n"
+      if (#a > 0) then
+         local ct = ColumnTable:new{tbl=a, gap=0, len=length, width=cwidth}
+         b[#b+1] = ct:build_tbl()
+         b[#b+1] = "\n"
+      end
    end
 
    if (next(legendT)) then
@@ -300,7 +340,7 @@ function List(...)
       b[#b+1] = "\nInactive Modules"
       b[#b+1] = msg2
       b[#b+1] = "\n"
-      local ct = ColumnTable:new{tbl=a,gap=0}
+      local ct = ColumnTable:new{tbl=a,gap=0, width = cwidth}
       b[#b+1] = ct:build_tbl()
       b[#b+1] = "\n"
    end
@@ -310,15 +350,15 @@ function List(...)
    if (#aa > 0) then
       b[#b+1] = concatTbl(aa,"")
    end
+
    shell:echo(concatTbl(b,""))
    dbg.fini("List")
 end
 
 
 ------------------------------------------------------------------------
--- Load_Try():  load modules from users but do not issue warnings
---              if the module is not there.
-
+-- Load modules from users but do not issue warnings if the module is
+-- not there.
 function Load_Try(...)
    local master = Master:master()
    local mt     = MT:mt()
@@ -331,19 +371,19 @@ function Load_Try(...)
 end
 
 ------------------------------------------------------------------------
--- Load_Usr():  load modules from users.  If a module name has
---              a minus sign in front of it then unload it.  Do that
---              before loading any other modules.  Also if the
---              shortName of a request module is already loaded then
---              unload it.  This way:
---                   $ module load foo/1.1; module load foo/1.3
---              the second load of "foo" will not load it twice.
---              Finally any successful loading of a module is registered
---              with "MT" so that when a user does the above commands
---              it won't get the swap message.
-
+-- Load modules from users.  If a module name has
+-- a minus sign in front of it then unload it.  Do that
+-- before loading any other modules.  Also if the
+-- shortName of a request module is already loaded then
+-- unload it.  This way:
+--
+--      $ module load foo/1.1; module load foo/1.3
+--
+-- the second load of "foo" will not load it twice.
+-- Finally any successful loading of a module is registered
+-- with "MT" so that when a user does the above commands
+-- it won't get the swap message.
 function Load_Usr(...)
-   LMOD_IGNORE_CACHE = true
    local master = Master:master()
    local mt     = MT:mt()
 
@@ -381,6 +421,7 @@ function Load_Usr(...)
       mcp           = MCP
       dbg.print{"Setting mcp to ", mcp:name(),"\n"}
       b             = mcp:load_usr(lA)
+
       if (haveWarnings()) then
          mcp.mustLoad()
       end
@@ -392,8 +433,8 @@ function Load_Usr(...)
 end
 
 --------------------------------------------------------------------------
--- Purge():  unload all loaded modules
-
+-- Unload all loaded modules.
+-- @param force If true then sticky modules are unloaded as well.
 function Purge(force)
    local master = Master:master()
    local mt     = MT:mt()
@@ -422,9 +463,8 @@ function Purge(force)
 end
 
 --------------------------------------------------------------------------
--- RecordCmd(): Write the current state of the module table to the
---              lmod-save directory.  This command should probably go away.
-
+-- Write the current state of the module table to the
+-- lmod-save directory.  This command should probably go away.
 function RecordCmd()
    dbg.start{"RecordCmd()"}
    local mt   = MT:mt()
@@ -448,12 +488,10 @@ function RecordCmd()
 end
 
 --------------------------------------------------------------------------
--- Refresh(): reload all loaded modules so that any alias or shell
---            functions are defined.  No other module commands are active.
---            This command exists so that sub-shells will have the aliases
---            defined.
-
-
+-- Reload all loaded modules so that any alias or shell
+-- functions are defined.  No other module commands are active.
+-- This command exists so that sub-shells will have the aliases
+-- defined.
 function Refresh()
    dbg.start{"Refresh()"}
    local master  = Master:master()
@@ -462,16 +500,25 @@ function Refresh()
 end
 
 --------------------------------------------------------------------------
--- Reset():  Reset all module commands back to the system defined default
---           value in env. var. LMOD_SYSTEM_DEFAULT_MODULES.  If that
---           variable is not defined then there are no default modules
---           and this command is the same as a purge.
-
+-- Reset all module commands back to the system defined default
+-- value in env. var. LMOD_SYSTEM_DEFAULT_MODULES.  If that
+-- variable is not defined then there are no default modules
+-- and this command is the same as a purge.
+-- @param msg If true then print resetting message.
 function Reset(msg)
    dbg.start{"Reset()"}
+   local default = os.getenv("LMOD_SYSTEM_DEFAULT_MODULES") or ""
+   if (default == "") then
+      io.stderr:write("\nThe system default contains no modules\n")
+      io.stderr:write("  (env var: LMOD_SYSTEM_DEFAULT_MODULES is empty)\n")
+      io.stderr:write("  No changes in loaded modules\n\n")
+      LmodErrorExit()
+      dbg.fini("Reset")
+      return
+   end
+
    local force = true
    Purge(force)
-   local default = os.getenv("LMOD_SYSTEM_DEFAULT_MODULES") or ""
    dbg.print{"default: \"",default,"\"\n"}
 
    default = default:trim()
@@ -482,13 +529,6 @@ function Reset(msg)
       io.stderr:write("Resetting modules to system default\n")
    end
 
-   if (default == "") then
-      io.stderr:write("\nThe system default contains no modules\n")
-      io.stderr:write("  (env var: LMOD_SYSTEM_DEFAULT_MODULES is empty)\n\n")
-      dbg.fini("Reset")
-      return
-   end
-
 
    local a = {}
    for m in default:split(":") do
@@ -502,12 +542,12 @@ function Reset(msg)
 end
 
 --------------------------------------------------------------------------
--- Restore(): Restore the state of the user's loaded modules original
---            state. If a user has a "default" then use that collection.
---            Otherwise do a "Reset()"
-
-function Restore(a)
-   dbg.start{"Restore(",a,")"}
+-- Restore the state of the user's loaded modules original
+-- state. If a user has a "default" then use that collection.
+-- Otherwise do a "Reset()"
+-- @param collection The user supplied collection name. If *nil* the use "default"
+function Restore(collection)
+   dbg.start{"Restore(",collection,")"}
 
    local msg
    local path
@@ -522,56 +562,66 @@ function Restore(a)
       sname   = "." .. sname
    end
 
-   if (a == nil) then
+   if (collection == nil) then
       path = pathJoin(os.getenv("HOME"), ".lmod.d", "default" .. sname)
       if (not isFile(path)) then
-         a = "system"
-         myName = a 
+         collection = "system"
+         myName = collection
       else
          myName = "default"
       end
-   elseif (a ~= "system") then
-      myName = sname
-      path   = pathJoin(os.getenv("HOME"), ".lmod.d", a .. sname)
+   elseif (collection ~= "system") then
+      myName = collection
+      path   = pathJoin(os.getenv("HOME"), ".lmod.d", collection .. sname)
       if (not isFile(path)) then
-         LmodError(" User module collection: \"",a,"\" does not exist.\n",
+         LmodError(" User module collection: \"",collection,"\" does not exist.\n",
                    " Try \"module savelist\" for possible choices.\n")
       end
    end
 
+   if (barefilename(myName):find("%.")) then
+      LmodError(" Collection names cannot have a `.' in the name.\n",
+                " Please rename \"", collection,"\"\n")
+   end
+
+
    local masterTbl = masterTbl()
 
-   if (a == "system" ) then
+   if (collection == "system" ) then
       msg = "system default" .. msgTail
    else
-      a   = a or "default"
-      msg = "user's ".. a .. msgTail
+      collection = collection or "default"
+      msg        = "user's ".. collection .. msgTail
    end
 
    if (masterTbl.quiet or masterTbl.initial) then
       msg = false
    end
 
-   if (a == "system" ) then
+   if (collection == "system" ) then
       Reset(msg)
    else
       local mt      = MT:mt()
-      local results = mt:getMTfromFile{fn=path, name=myName, msg=msg} or Reset(msg)
+      local results = mt:getMTfromFile{fn=path, name=myName, msg=msg}
+      if (not results and collection == "default") then
+         Reset(msg)
+      end
    end
 
+   hook.apply("restore", {collection=collection, name=myName, fn=path})
+
    dbg.fini("Restore")
 end
 
 --------------------------------------------------------------------------
--- Save(): Save the current list of modules to whatever name the user
---         gave on the command line or "default" if there is none.
---         Note that this function checks to see if any of the currently
---         loaded modules mix load statements with the setting of
---         environment variables.  In that case, Lmod will not save the
---         state of the modules into a collection.  The reasons are
---         complicated but a "manager" module file is "fake" loaded so
---         that any setenv or prepend_path commands will not be executed.
-
+-- Save the current list of modules to whatever name the user
+-- gave on the command line or "default" if there is none.
+-- Note that this function checks to see if any of the currently
+-- loaded modules mix load statements with the setting of
+-- environment variables.  In that case, Lmod will not save the
+-- state of the modules into a collection.  The reasons are
+-- complicated but a "manager" module file is "fake" loaded so
+-- that any setenv or prepend_path commands will not be executed.
 function Save(...)
    local masterTbl = masterTbl()
    local mt        = MT:mt()
@@ -588,6 +638,12 @@ function Save(...)
       sname   = "." .. sname
    end
 
+   if (barefilename(a):find("%.")) then
+      LmodWarning("It is illegal to have a `.' in a collection name.  Please choose another name: ",a)
+      dbg.fini("Save")
+      return
+   end
+
    if (a == "system") then
       LmodWarning("The named collection 'system' is reserved. Please choose another name.\n")
       dbg.fini("Save")
@@ -633,8 +689,7 @@ end
 
 
 --------------------------------------------------------------------------
--- SaveList(): report to the user all the named collections he/she has.
-
+-- Report to the user all the named collections he/she has.
 function SaveList(...)
    local mt        = MT:mt()
    local path      = pathJoin(os.getenv("HOME"), LMODdir)
@@ -642,6 +697,7 @@ function SaveList(...)
    local a         = {}
    local b         = {}
    local shell     = Master:master().shell
+   local cwidth    = masterTbl.rt and LMOD_COLUMN_TABLE_WIDTH or TermWidth()
 
    findNamedCollections(b,path)
    if (masterTbl.terse) then
@@ -651,7 +707,7 @@ function SaveList(...)
          if (i) then
             name = name:sub(j+2)
          end
-         io.stderr:write(name,"\n")
+         shell:echo(name.."\n")
       end
       return
    end
@@ -667,10 +723,15 @@ function SaveList(...)
       a[#a+1] = cstr .. name
    end
 
-   local b = {}
+   b            = {}
+   local msgHdr = ""
+   if (LMOD_SYSTEM_NAME) then
+      msgHdr = "(For LMOD_SYSTEM_NAME = \""..LMOD_SYSTEM_NAME.."\")"
+   end
+
    if (#a > 0) then
-      b[#b+1]  = "Named collection list:\n"
-      local ct = ColumnTable:new{tbl=a,gap=0}
+      b[#b+1]  = "Named collection list ".. msgHdr..":\n"
+      local ct = ColumnTable:new{tbl=a,gap=0,width=cwidth}
       b[#b+1]  = ct:build_tbl()
       b[#b+1]  = "\n"
       shell:echo(concatTbl(b,""))
@@ -680,8 +741,7 @@ function SaveList(...)
 end
 
 --------------------------------------------------------------------------
--- SearchCmd(): Point users to either spider or keyword
-
+-- Point users to either spider or keyword
 function SearchCmd(...)
    local s = concatTbl({...}, " ")
    io.stderr:write("\"module search\" does not exist. To list all possible modules do: \n",
@@ -690,14 +750,10 @@ function SearchCmd(...)
                    "   module keyword ",s,"\n")
 end
 
-
-
-
 --------------------------------------------------------------------------
--- Show(): use the show mode of MasterControl to list the active Lmod
---         commands in a module file.  Note that it is always in Lua
---         even if the modulefile is written in TCL.
-
+-- Use the show mode of MasterControl to list the active Lmod
+-- commands in a module file.  Note that it is always in Lua
+-- even if the modulefile is written in TCL.
 function Show(...)
    local master = Master:master()
    dbg.start{"Show(", concatTbl({...},", "),")"}
@@ -706,7 +762,7 @@ function Show(...)
    local borderStr = banner:border(0)
 
    prtHdr       = function()
-                     local a = {}  
+                     local a = {}
                      a[#a+1] = borderStr
                      a[#a+1] = "   "
                      a[#a+1] = ModuleFn
@@ -719,21 +775,18 @@ function Show(...)
 end
 
 --------------------------------------------------------------------------
--- SpiderCmd(): Do a level 0 spider report if there are no command line
---              arguments.  Otherwise do a spider search to generate a
---              level 1 or level 2 report on particular modules.
-
+-- Do a level 0 spider report if there are no command line
+-- arguments.  Otherwise do a spider search to generate a
+-- level 1 or level 2 report on particular modules.
 function SpiderCmd(...)
    dbg.start{"SpiderCmd(", concatTbl({...},", "),")"}
-   local cache     = Cache:cache()
-   local shell     = Master:master().shell
-   local moduleT   = cache:build()
-   local masterTbl = masterTbl()
-   local dbT       = {}
+   local cache       = Cache:cache{buildCache=true}
+   local shell       = Master:master().shell
+   local masterTbl   = masterTbl()
+   local moduleT,dbT = cache:build()
+   local spider      = Spider:new()
    local s
    local srch
-   local spider    = Spider:new()
-   spider:buildSpiderDB({"default"},moduleT, dbT)
 
    local arg = pack(...)
 
@@ -750,7 +803,7 @@ function SpiderCmd(...)
    end
 
    if (masterTbl.terse) then
-      io.stderr:write(s,"\n")
+      shell:echo(s.."\n")
    else
       local a = {}
       a[#a+1] = s
@@ -762,12 +815,10 @@ function SpiderCmd(...)
 end
 
 --------------------------------------------------------------------------
---  Swap(): Swap one module for another.  That is unload the first
---          and load the second.  If the second modulefile is successfully
---          loaded then it is registered with MT so that it won't be
---          reported in a swap message.
-
-
+--  Swap one module for another.  That is unload the first
+--  and load the second.  If the second modulefile is successfully
+--  loaded then it is registered with MT so that it won't be
+--  reported in a swap message.
 function Swap(...)
    local a = select(1, ...) or ""
    local b = select(2, ...) or ""
@@ -796,14 +847,15 @@ function Swap(...)
    mA[1]         = MName:new("load",b)
    local aa = mcp:load_usr(mA)
    if (not aa[1]) then
-      LmodError("Swap failed.\n")
+      mcp.mustLoad()
    end
 
    ------------------------------------------------------
    -- Register user loads so that Karl will be happy.
 
-   local mname = mA[1]
-   local sn    = mname:sn()
+
+   mname       = mA[1]
+   sn          = mname:sn()
    local usrN  = (not masterTbl().latest) and b or mt:fullName(sn)
    mt:userLoad(sn,usrN)
    mcp = mcp_old
@@ -812,8 +864,7 @@ function Swap(...)
 end
 
 --------------------------------------------------------------------------
---  TableList(): list the loaded modules in a lua table
-
+--  list the loaded modules in a lua table
 function TableList()
    dbg.start{"TableList()"}
    local mt = MT:mt()
@@ -833,18 +884,16 @@ function TableList()
 end
 
 --------------------------------------------------------------------------
---  Update(): reload all modules.
-
+--  Reload all modules.
 function Update()
    local master = Master:master()
    master:reloadAll()
 end
 
 --------------------------------------------------------------------------
---  Use(): add a directory to MODULEPATH and LMOD_DEFAULT_MODULEPATH.
---         Note that this causes all the modules to be reviewed and
---         possibly reloaded if a module.
-
+--  Add a directory to MODULEPATH and LMOD_DEFAULT_MODULEPATH.
+--  Note that this causes all the modules to be reviewed and
+--  possibly reloaded if a module.
 function Use(...)
    local mt  = MT:mt()
    local a = {}
@@ -868,15 +917,15 @@ function Use(...)
       end
       iarg = iarg + 1
    end
-   local nodups = true
    for _,v in ipairs(a) do
-      v = abspath(v)
-      if (v) then
-         op(MCP, { ModulePath,  v, delim = ":", nodups=nodups, priority=priority })
-         op(MCP, { DfltModPath, v, delim = ":", nodups=nodups, priority=priority })
+      dbg.print{"v: ",v,", isDir(v): ",isDir(v),"\n"}
+      if (isDir(v)) then
+         op(MCP, { ModulePath,  v, delim = ":", nodups=true, priority=priority })
+         op(MCP, { DfltModPath, v, delim = ":", nodups=true, priority=priority })
       end
    end
-
+   local master    = Master:master()
+   master.reloadAll()
 
    mt:buildBaseMpathA(varTbl[DfltModPath]:expand())
    mt:buildMpathA(varTbl[ModulePath]:expand())
@@ -884,10 +933,9 @@ function Use(...)
 end
 
 --------------------------------------------------------------------------
---  UnUse(): remove a directory from both MODULEPATH and
---           LMOD_DEFAULT_MODULEPATH.  Note that all currently loaded
---           modules reviewed and possibly reloaded or made inactive.
-
+--  Remove a directory from both MODULEPATH and
+--  LMOD_DEFAULT_MODULEPATH.  Note that all currently loaded
+--  modules reviewed and possibly reloaded or made inactive.
 function UnUse(...)
    local mt  = MT:mt()
    dbg.start{"UnUse(", concatTbl({...},", "),")"}
@@ -904,8 +952,7 @@ function UnUse(...)
 end
 
 --------------------------------------------------------------------------
---  UnLoad():  unload all requested modules
-
+-- Unload all requested modules
 function UnLoad(...)
    dbg.start{"UnLoad(",concatTbl({...},", "),")"}
    MCP:unload_usr(MName:buildA("mt", ...))
@@ -913,8 +960,7 @@ function UnLoad(...)
 end
 
 --------------------------------------------------------------------------
---  Whatis(): Run whatis on all request modules given the the command line.
-
+-- Run whatis on all request modules given the the command line.
 function Whatis(...)
    prtHdr    = function () return "" end
    Access("whatis",...)
