@@ -40,38 +40,61 @@
 --------------------------------------------------------------------------
 
 BaseShell       = {}
-Pager           = "@path_to_pager@"
-s_prependBlock  = "@prepend_block@"
-prepend_order   = false
 banner          = false
 
-------------------------------------------------------------------------
--- Extract directory location of "lmod" command and add it
--- to the lua search path
-------------------------------------------------------------------------
 
 ------------------------------------------------------------------------
 -- Use command name to add the command directory to the package.path
 ------------------------------------------------------------------------
-local LuaCommandName = arg[0]
-local i,j = LuaCommandName:find(".*/")
-local LuaCommandName_dir = "./"
-if (i) then
-   LuaCommandName_dir = LuaCommandName:sub(1,j)
-   LuaCommandName     = LuaCommandName:sub(j+1)
+
+local sys_lua_path = "@sys_lua_path@"
+if (sys_lua_path:sub(1,1) == "@") then
+   sys_lua_path = package.path
 end
 
-package.path = LuaCommandName_dir .. "?.lua;"       ..
-               LuaCommandName_dir .. "../tools/?.lua;"  ..
-               LuaCommandName_dir .. "../shells/?.lua;" ..
-               LuaCommandName_dir .. "?/init.lua;"  ..
-               package.path
+local sys_lua_cpath = "@sys_lua_cpath@"
+if (sys_lua_cpath:sub(1,1) == "@") then
+   sys_lua_cpath = package.cpath
+end
 
+package.path   = sys_lua_path
+package.cpath  = sys_lua_cpath
+
+local arg_0    = arg[0]
+local posix    = require("posix")
+local readlink = posix.readlink
+local stat     = posix.stat
+
+local st       = stat(arg_0)
+while (st.type == "link") do
+   arg_0 = readlink(arg_0)
+   st    = stat(arg_0)
+end
+
+local ia,ja = arg_0:find(".*/")
+local LuaCommandName_dir = "./"
+if (ia) then
+   LuaCommandName_dir = arg_0:sub(1,ja)
+   LuaCommandName     = arg_0:sub(ja+1)
+end
+
+package.path  = LuaCommandName_dir .. "?.lua;"       ..
+                LuaCommandName_dir .. "../tools/?.lua;"  ..
+                LuaCommandName_dir .. "../shells/?.lua;" ..
+                LuaCommandName_dir .. "?/init.lua;"  ..
+                sys_lua_path
 package.cpath = LuaCommandName_dir .. "../lib/?.so;"..
-                package.cpath
+                sys_lua_cpath
 
 
 require("strict")
+
+--------------------------------------------------------------------------
+-- Return the path to the Lmod program
+function cmdDir()
+   return LuaCommandName_dir
+end
+
 require("myGlobals")
 
 local term     = false
@@ -79,10 +102,9 @@ if (pcall(require, "term")) then
    term = require("term")
 end
 
-function cmdDir()
-   return LuaCommandName_dir
-end
 
+--------------------------------------------------------------------------
+-- Return this program's name.
 function cmdName()
    return LuaCommandName
 end
@@ -90,29 +112,10 @@ end
 local getenv = os.getenv
 local rep    = string.rep
 
-function set_prepend_order()
-   local ansT = {
-      no      = "reverse",
-      reverse = "reverse",
-      normal  = "normal",
-      yes     = "normal",
-   }
-
-   local order = ansT[getenv("LMOD_PREPEND_BLOCK") or s_prependBlock] or "normal"
-   if (order == "normal") then
-      prepend_order = function (n)
-         return n, 1, -1
-      end
-   else
-      prepend_order = function (n)
-         return 1, n, 1
-      end
-   end
-end
-
-
+local BuildFactory = require("BuildFactory")
+BuildFactory:master()
 require("utils")
-build_epoch()          -- build the epoch function
+
 require("pager")
 require("fileOps")
 MasterControl = require("MasterControl")
@@ -128,35 +131,30 @@ MT            = require("MT")
 Exec          = require("Exec")
 
 local BeautifulTbl = require('BeautifulTbl')
+local ReadLmodRC   = require('ReadLmodRC')
 local dbg          = require("Dbg"):dbg()
 local Banner       = require("Banner")
-local Timer        = require("Timer")
 local Version      = require("Version")
 local concatTbl    = table.concat
 local max          = math.max
-local unpack       = unpack or table.unpack
-local timer        = Timer:timer()
-
-function set_duplication()
-   local dups = LMOD_DUPLICATE_PATHS:lower()
-   if (dups == "yes") then
-      dbg.print{"Allowing duplication in paths\n"}
-      allow_dups = function (dupsIn)
-         return dupsIn
-      end
-   else
-      dbg.print{"No duplication allowed in paths\n"}
-      allow_dups = function (dupsIn)
-         return false
-      end
-   end
-end
-
+local unpack       = (_VERSION == "Lua 5.1") and unpack or table.unpack
+local timer        = require("Timer"):timer()
+local hook         = require("Hook")
 
+--------------------------------------------------------------------------
+-- Use the *propT* table to colorize the module name when requested by
+-- *propT*.
+-- @param style How to colorize
+-- @param moduleName The module name
+-- @param propT The property table
+-- @param legendT The legend table.  A key-value pairing of keys to descriptions.
+-- @return An array of colorized strings
 function colorizePropA(style, moduleName, propT, legendT)
    local resultA      = { moduleName }
-   local propDisplayT = getPropT()
+   local readLmodRC   = ReadLmodRC:singleton()
+   local propDisplayT = readLmodRC:propT()
    local iprop        = 0
+   local pA           = {}
    propT              = propT or {}
 
    for kk,vv in pairsByKeys(propDisplayT) do
@@ -172,28 +170,38 @@ function colorizePropA(style, moduleName, propT, legendT)
 
          table.sort(propA);
          local n = concatTbl(propA,":")
-
          if (vv.displayT[n]) then
             result     = vv.displayT[n][style]
+            if (result:sub( 1, 1) == "(" and result:sub(-1,-1) == ")") then
+               result  = result:sub(2,-2)
+            end
             color      = vv.displayT[n].color
             local k    = colorize(color,result)
             legendT[k] = vv.displayT[n].doc
          end
       end
       local s             = colorize(color,result)
-      resultA[#resultA+1] = s
+      if (result:len() > 0) then
+         pA[#pA+1] = s
+      end
+   end
+   if (#pA > 0) then
+      resultA[#resultA+1] = concatTbl(pA,",")
    end
    return resultA
 end
 
 
-s_Usage = false
 
+s_Usage = false
+--------------------------------------------------------------------------
+-- Build the lmod usage message and store in *s_Usage*.
 function Usage()
    if (s_Usage) then
       return s_Usage
    end
-   local website = colorize("red","http://www.tacc.utexas.edu/tacc-projects/lmod")
+   local website = colorize("red","http://lmod.readthedocs.org/")
+   local webBR   = colorize("red","http://lmod.readthedocs.io/en/latest/075_bug_reporting.html")
    local line    = banner:border(2)
    local a = {}
    a[#a+1] = { "module [options] sub-command [args ...]" }
@@ -236,16 +244,17 @@ function Usage()
    a[#a+1] = { "" }
    a[#a+1] = { "Handling a collection of modules:"}
    a[#a+1] = { "--------------------------------" }
-   a[#a+1] = { "  save | s",    "",       "Save the current list of modules to a user defined \"default\"."}
+   a[#a+1] = { "  save | s",    "",       "Save the current list of modules to a user defined \"default\" collection."}
    a[#a+1] = { "  save | s",    "name",   "Save the current list of modules to \"name\" collection."}
+   a[#a+1] = { "  reset",     "",        "The same as \"restore system\""}
    a[#a+1] = { "  restore | r", "",       "Restore modules from the user's \"default\" or system default."}
    a[#a+1] = { "  restore | r", "name",   "Restore modules from \"name\" collection."}
    a[#a+1] = { "  restore",     "system", "Restore module state to system defaults."}
    a[#a+1] = { "  savelist",    "",       "List of saved collections."}
+   a[#a+1] = { "  describe | mcc",  "name",  "Describe the contents of a module collection."}
    a[#a+1] = { "" }
    a[#a+1] = { "Deprecated commands:"}
    a[#a+1] = { "--------------------"}
-   a[#a+1] = { "  reset",     "",        "The same as \"restore system\""}
    a[#a+1] = { "  getdefault", "[name]", "load name collection of modules or "..
                                          "user's \"default\" if no name given."}
    a[#a+1] = { "",            "",        "===> Use \"restore\" instead  <===="}
@@ -266,11 +275,14 @@ function Usage()
                                       "properties and warning in color."}
    a[#a+1] = { "" }
    a[#a+1] = { line}
-   a[#a+1] = { "The following guides are at "..website}
+   a[#a+1] = { "Lmod Web Sites"}
    a[#a+1] = { "" }
-   a[#a+1] = { "  User Guide                 - How to use."}
-   a[#a+1] = { "  Advance User Guide         - How to create you own modules."}
-   a[#a+1] = { "  System Administrator Guide - How to install Lmod on your own system."}
+   a[#a+1] = { "  Documentation:    http://lmod.readthedocs.org"}
+   a[#a+1] = { "  Github:           https://github.com/TACC/Lmod"}
+   a[#a+1] = { "  Sourceforge:      https://lmod.sf.net"}
+   a[#a+1] = { "  TACC Homepage:    https://www.tacc.utexas.edu/research-development/tacc-projects/lmod"}
+   a[#a+1] = { "" }
+   a[#a+1] = { "  To report a bug please read "..webBR }
    a[#a+1] = { line }
 
 
@@ -286,6 +298,8 @@ end
 CmdLineUsage = "Usage: module [options] sub-command [args ...]"
 
 
+--------------------------------------------------------------------------
+-- Build the version string.
 function version()
    local v = {}
    v[#v + 1] = "\nModules based on Lua: Version " .. Version.name().."\n"
@@ -301,12 +315,16 @@ local Options      = require("Options")
 local Spider       = require("Spider")
 local Var          = require("Var")
 
+--------------------------------------------------------------------------
+-- A place holder function.  This should never be called.
 function None()
    print ("None")
-   FooBar=1
 end
 
 
+--------------------------------------------------------------------------
+-- Register local variables into the *varTbl* table.
+-- @param localvarA
 local function localvar(localvarA)
    for _, v in ipairs(localvarA) do
       local i = v:find("=")
@@ -324,131 +342,94 @@ end
 
 
 
-add	     = None
-rm	     = None
-use          = None
-unuse        = None
-prtHdr       = None
-avail        = Avail
-list         = List
-
+prtHdr     = None
 ModuleName = ""
 ModuleFn   = ""
-
-
-
+if (prtHdr ~= None) then
+   prtHdr()
+end
+--------------------------------------------------------------------------
+-- The main function of Lmod.  The lmod program always starts here.
 function main()
    local epoch        = epoch
    local t1           = epoch()
-   local loadTbl      = { name = "load",        checkMPATH = true,  cmd = Load_Usr    }
-   local tryAddTbl    = { name = "try-add",     checkMPATH = true,  cmd = Load_Try    }
-   local unloadTbl    = { name = "unload",      checkMPATH = true,  cmd = UnLoad      }
-   local refreshTbl   = { name = "refresh",     checkMPATH = false, cmd = Refresh     }
-   local swapTbl      = { name = "swap",        checkMPATH = true,  cmd = Swap        }
-   local purgeTbl     = { name = "purge",       checkMPATH = true,  cmd = Purge       }
-   local resetTbl     = { name = "reset",       checkMPATH = true,  cmd = Reset       }
-   local availTbl     = { name = "avail",       checkMPATH = false, cmd = Avail       }
-   local listTbl      = { name = "list",        checkMPATH = false, cmd = List        }
-   local tblLstTbl    = { name = "tablelist",   checkMPATH = false, cmd = TableList   }
-   local helpTbl      = { name = "help",        checkMPATH = false, cmd = Help        }
-   local whatisTbl    = { name = "whatis",      checkMPATH = false, cmd = Whatis      }
-   local showTbl      = { name = "show",        checkMPATH = false, cmd = Show        }
-   local useTbl       = { name = "use",         checkMPATH = true,  cmd = Use         }
-   local unuseTbl     = { name = "unuse",       checkMPATH = true,  cmd = UnUse       }
-   local updateTbl    = { name = "update",      checkMPATH = true,  cmd = Update      }
-   local keywordTbl   = { name = "keyword",     checkMPATH = false, cmd = Keyword     }
-   local saveTbl      = { name = "save",        checkMPATH = false, cmd = Save        }
-   local gdTbl        = { name = "getDefault",  checkMPATH = false, cmd = GetDefault  }
-   local restoreTbl   = { name = "restore",     checkMPATH = false, cmd = Restore     }
-   local savelistTbl  = { name = "savelist",    checkMPATH = false, cmd = SaveList    }
-   local spiderTbl    = { name = "spider",      checkMPATH = true,  cmd = SpiderCmd   }
-   local searchTbl    = { name = "search",      checkMPATH = false, cmd = SearchCmd   }
-   local recordTbl    = { name = "record",      checkMPATH = false, cmd = RecordCmd   }
 
-   local cmdTbl = {
-      ["try-add"]  = tryAddTbl,
-      ["try-load"] = tryAddTbl,
-      add          = loadTbl,
-      apropos      = keywordTbl,
-      av           = availTbl,
-      avail        = availTbl,
-      del          = unloadTbl,
-      delete       = unloadTbl,
-      dis          = showTbl,
-      display      = showTbl,
-      era          = unloadTbl,
-      erase        = unloadTbl,
-      gd           = gdTbl,
-      getd         = gdTbl,
-      getdefault   = gdTbl,
-      help         = helpTbl,
-      key          = keywordTbl,
-      keyword      = keywordTbl,
-      ld           = savelistTbl,
-      li           = listTbl,
-      list         = listTbl,
-      listdefault  = savelistTbl,
-      savelist     = savelistTbl,
-      lo           = loadTbl,
-      load         = loadTbl,
-      purge        = purgeTbl,
-      record       = recordTbl,
-      refr         = updateTbl,
-      refresh      = refreshTbl,
-      reload       = updateTbl,
-      remov        = unloadTbl,
-      remove       = unloadTbl,
-      reset        = resetTbl,
-      restore      = restoreTbl,
-      r            = restoreTbl,
-      rm           = unloadTbl,
-      s            = saveTbl,
-      save         = saveTbl,
-      sd           = saveTbl,
-      search       = searchTbl,
-      setd         = saveTbl,
-      setdefault   = saveTbl,
-      show         = showTbl,
-      spider       = spiderTbl,
-      sw           = swapTbl,
-      swap         = swapTbl,
-      switch       = swapTbl,
-      tablelist    = tblLstTbl,
-      unlo         = unloadTbl,
-      unload       = unloadTbl,
-      unuse        = unuseTbl,
-      update       = updateTbl,
-      use          = useTbl,
-      wh           = whatisTbl,
-      whatis       = whatisTbl,
+   local availTbl     = { name = "avail",       checkMPATH = false, cmd = Avail         }
+   local gdTbl        = { name = "getDefault",  checkMPATH = false, cmd = GetDefault    }
+   local helpTbl      = { name = "help",        checkMPATH = false, cmd = Help          }
+   local keywordTbl   = { name = "keyword",     checkMPATH = false, cmd = Keyword       }
+   local listTbl      = { name = "list",        checkMPATH = false, cmd = List          }
+   local loadTbl      = { name = "load",        checkMPATH = true,  cmd = Load_Usr      }
+   local mcTbl        = { name = "describe",    checkMPATH = false, cmd = CollectionLst }
+   local purgeTbl     = { name = "purge",       checkMPATH = true,  cmd = Purge         }
+   local recordTbl    = { name = "record",      checkMPATH = false, cmd = RecordCmd     }
+   local refreshTbl   = { name = "refresh",     checkMPATH = false, cmd = Refresh       }
+   local resetTbl     = { name = "reset",       checkMPATH = true,  cmd = Reset         }
+   local restoreTbl   = { name = "restore",     checkMPATH = false, cmd = Restore       }
+   local saveTbl      = { name = "save",        checkMPATH = false, cmd = Save          }
+   local savelistTbl  = { name = "savelist",    checkMPATH = false, cmd = SaveList      }
+   local searchTbl    = { name = "search",      checkMPATH = false, cmd = SearchCmd     }
+   local showTbl      = { name = "show",        checkMPATH = false, cmd = Show          }
+   local spiderTbl    = { name = "spider",      checkMPATH = true,  cmd = SpiderCmd     }
+   local swapTbl      = { name = "swap",        checkMPATH = true,  cmd = Swap          }
+   local tblLstTbl    = { name = "tablelist",   checkMPATH = false, cmd = TableList     }
+   local tryAddTbl    = { name = "try-add",     checkMPATH = true,  cmd = Load_Try      }
+   local unloadTbl    = { name = "unload",      checkMPATH = true,  cmd = UnLoad        }
+   local unuseTbl     = { name = "unuse",       checkMPATH = true,  cmd = UnUse         }
+   local updateTbl    = { name = "update",      checkMPATH = true,  cmd = Update        }
+   local useTbl       = { name = "use",         checkMPATH = true,  cmd = Use           }
+   local whatisTbl    = { name = "whatis",      checkMPATH = false, cmd = Whatis        }
+
+   local lmodCmdA = {
+      {'^ad'      , loadTbl       },
+      {'^ap'      , keywordTbl    },
+      {'^av'      , availTbl      },
+      {'^del'     , unloadTbl     },
+      {'^des'     , mcTbl         },
+      {'^dis'     , showTbl       },
+      {'^era'     , unloadTbl     },
+      {'^gd'      , gdTbl         },
+      {'^getd'    , gdTbl         },
+      {'^h'       , helpTbl       },
+      {'^k'       , keywordTbl    },
+      {'^ld'      , savelistTbl   },
+      {'^listd'   , savelistTbl   },
+      {'^lo'      , loadTbl       },
+      {'^l'       , listTbl       },
+      {'^mc'      , mcTbl         },
+      {'^pu'      , purgeTbl      },
+      {'^rec'     , recordTbl     },
+      {'^refr'    , refreshTbl    },
+      {'^rel'     , updateTbl     },
+      {'^rem'     , unloadTbl     },
+      {'^rese'    , resetTbl      },
+      {'^rm'      , unloadTbl     },
+      {'^r'       , restoreTbl    },
+      {'^savel'   , savelistTbl   },
+      {'^sd'      , saveTbl       },
+      {'^sea'     , searchTbl     },
+      {'^setd'    , saveTbl       },
+      {'^sh'      , showTbl       },
+      {'^sl'      , savelistTbl   },
+      {'^sp'      , spiderTbl     },
+      {'^sw'      , swapTbl       },
+      {'^s'       , saveTbl       },
+      {'^table'   , tblLstTbl     },
+      {'^try'     , tryAddTbl     },
+      {'^unuse$'  , unuseTbl      },
+      {'^un'      , unloadTbl     },
+      {'^up'      , updateTbl     },
+      {'^use$'    , useTbl        },
+      {'^w'       , whatisTbl     },
    }
 
    MCP = MasterControl.build("load")
    mcp = MasterControl.build("load")
 
-   -------------------------------------------------------------------
-   -- Is io.stderr connected to a tty or not?
-   -- Setup output and pager routines
-   colorize = plain
-   pager    = bypassPager
-   local connectedTerm = false
 
-   if (term and getenv("TERM")) then
-      if (term.isatty(io.stderr)) then
-         colorize      = full_colorize
-         pager         = buildPager()
-         connectedTerm = true
-      end
-   end
-
-   local lmod_colorize = getenv("LMOD_COLORIZE") or "@colorize@"
-   if (lmod_colorize:lower() ~= "yes") then
-      colorize = plain
-   end
-
-   if (lmod_colorize:lower() == "force") then
-      colorize = full_colorize
-   end
+   ------------------------------------------------------------
+   -- Chose parseVersion:
+   parseVersion = buildParseVersion()
 
 
    dbg.set_prefix(colorize("red","Lmod"))
@@ -464,14 +445,8 @@ function main()
    local arg_str   = concatTbl(arg," ")
    local masterTbl = masterTbl()
 
-   set_prepend_order()    -- Chose prepend_path order normal/reverse
    setenv_lmod_version()  -- Push Lmod version into environment
 
-
-   set_duplication()         -- Chose how to handle duplicate entries in a path.
-   build_accept_functions()  -- build the accept functions to allow or ignore TCL mfiles
-   readRC()
-
    ------------------------------------------------------------------------
    --  The StandardPackage is where Lmod registers hooks.  Sites may
    --  override the hook functions in SitePackage.
@@ -503,7 +478,7 @@ function main()
    localvar(masterTbl.localvarA)
 
    banner        = Banner:banner()
-   local cmdName = masterTbl.pargs[1]
+   local usrCmd = masterTbl.pargs[1]
    table.remove(masterTbl.pargs,1)
 
    if (masterTbl.debug > 0 or masterTbl.dbglvl) then
@@ -513,26 +488,15 @@ function main()
       dbg:activateDebug(dbgLevel)
    end
    dbg.start{"lmod(", arg_str,")"}
-   dbg.print{"Lmod Version: ",Version.name(),"\n"}
-   dbg.print{"package.path: ",package.path,"\n"}
-
-   ------------------------------------------------------------
-   -- Must output local variables even when there is the command
-   -- is not a valid command
-   --
-   -- So set [[checkMPATH]] to false by default and re-define when there
-   -- is a valid command:
-
-   local checkMPATH = false
-   if (cmdTbl[cmdName] ) then
-      checkMPATH = cmdTbl[cmdName].checkMPATH
+   if (dbg.active()) then
+      dbg.print{"Date: ",os.date(),"\n"}
+      dbg.print{"Hostname: ",posix.uname("%n"),"\n"}
+      dbg.print{"System: ",posix.uname("%s")," ",posix.uname("%r"),"\n"}
+      dbg.print{"Version: ",posix.uname("%v"),"\n"}
+      dbg.print{"Lmod Version: ",Version.name(),"\n"}
+      dbg.print{"package.path: ",package.path,"\n"}
    end
 
-   if (LMOD_RTM_TESTING) then
-      os.exit(0)
-   end
-
-
    -- dumpversion and quit if requested.
 
    if (masterTbl.dumpversion) then
@@ -540,6 +504,18 @@ function main()
       os.exit(0)
    end
 
+   -- gitversion and quit if requested.
+   if (masterTbl.gitversion) then
+      local gitV = Version.git()
+      if (gitV == "") then
+         gitV = Version.tag()
+      else
+         gitV = gitV:match('%((.*)%)')
+      end
+      io.stderr:write(gitV,"\n")
+      os.exit(0)
+   end
+
    -- print version and quit if requested.
    if (masterTbl.version) then
       io.stderr:write(version())
@@ -558,6 +534,42 @@ function main()
       os.exit(0)
    end
 
+   -- dump Configuration in json and quit.
+   if (masterTbl.configjson) then
+      local Configuration = require("Configuration")
+      local configuration = Configuration:configuration()
+      local a = configuration:report_json()
+      io.stderr:write(a)
+      os.exit(0)
+   end
+
+   ------------------------------------------------------------
+   -- Search for command, quit if command is unknown.
+   local cmdT = false
+   if (usrCmd) then
+      for _, v in ipairs(lmodCmdA) do
+         if (usrCmd:find(v[1])) then
+            cmdT = v[2]
+            break
+         end
+      end
+   end
+
+   ------------------------------------------------------------
+   -- Must output local variables even when there is the command
+   -- is not a valid command
+   --
+   -- So set [[checkMPATH]] to false by default and re-define when there
+   -- is a valid command:
+
+   local checkMPATH = (cmdT) and cmdT.checkMPATH or false
+
+   if (LMOD_RTM_TESTING) then
+      os.exit(0)
+   end
+
+
+   hook.apply("startup", usrCmd)
 
    -- Create the [[master]] object
    local master = Master:master(checkMPATH)
@@ -582,18 +594,13 @@ function main()
       os.exit(0)
    end
 
-   -- Now quit if command is unknown.
-
-   if (cmdTbl[cmdName] == nil) then
+   if (not cmdT) then
       io.stderr:write(version())
       io.stderr:write(Usage(),"\n")
       LmodErrorExit()
-   end
-
-   if (cmdTbl[cmdName]) then
-      local cmd = cmdTbl[cmdName].cmd
-      cmdName   = cmdTbl[cmdName].name
-      dbg.print{"cmd name: ", cmdName,"\n"}
+   else
+      local cmd  = cmdT.cmd
+      dbg.print{"cmd name: ", cmdT.name,"\n"}
       cmd(unpack(masterTbl.pargs))
    end
 
@@ -601,6 +608,10 @@ function main()
    -- a new one.
    mt = MT:mt()
 
+   -- Report any admin messages associated with loads
+   -- Note that is safe to run every time.
+   mcp:reportAdminMsgs()
+
    -- Report any changes (worth reporting from original MT)
    if (not quiet()) then
       mt:reportChanges()
@@ -608,18 +619,22 @@ function main()
 
    -- Store the Module table in "_ModuleTable_" env. var.
    local n        = mt:name()
-   local oldValue = getMT() or ""
    local value    = mt:serializeTbl()
-   varTbl[n] = Var:new(n)
+   varTbl[n]      = Var:new(n)
    varTbl[n]:set(value)
 
+   -- Run any registered function for the Exit Hook
+   ExitHookA.apply()
+
    dbg.fini("Lmod")
 
    -- Output all newly created path and variables.
    master.shell:expand(varTbl)
 
-   -- Expand any shell command registered.
-   Exec:exec():expand()
+   -- Expand any shell commands registered for "real" shells: Csh/Bash/Fish/...
+   if (master.shell:real_shell()) then
+      Exec:exec():expand()
+   end
 
    local t2 = epoch()
    timer:deltaT("main", t2 - t1)
