@@ -39,22 +39,51 @@
 --
 --------------------------------------------------------------------------
 
-local LuaCommandName = arg[0]
-local i,j = LuaCommandName:find(".*/")
-local LuaCommandName_dir = "./"
-if (i) then
-   LuaCommandName_dir = LuaCommandName:sub(1,j)
+local sys_lua_path = "@sys_lua_path@"
+if (sys_lua_path:sub(1,1) == "@") then
+   sys_lua_path = package.path
 end
-package.path = LuaCommandName_dir .. "../tools/?.lua;"  ..
-               LuaCommandName_dir .. "../shells/?.lua;" ..
-               LuaCommandName_dir .. "?.lua;"           ..
-               package.path
+
+local sys_lua_cpath = "@sys_lua_cpath@"
+if (sys_lua_cpath:sub(1,1) == "@") then
+   sys_lua_cpath = package.cpath
+end
+
+package.path   = sys_lua_path
+package.cpath  = sys_lua_cpath
+
+local arg_0    = arg[0]
+local posix    = require("posix")
+local readlink = posix.readlink
+local stat     = posix.stat
+
+local st       = stat(arg_0)
+while (st.type == "link") do
+   arg_0 = readlink(arg_0)
+   st    = stat(arg_0)
+end
+
+local ia,ja = arg_0:find(".*/")
+local LuaCommandName_dir = "./"
+if (ia) then
+   LuaCommandName_dir = arg_0:sub(1,ja)
+end
+
+package.path  = LuaCommandName_dir .. "../tools/?.lua;"  ..
+                LuaCommandName_dir .. "../shells/?.lua;" ..
+                LuaCommandName_dir .. "?.lua;"           ..
+                sys_lua_path
+package.cpath = sys_lua_cpath
 
 function cmdDir()
    return LuaCommandName_dir
 end
 
 require("strict")
+
+local BuildFactory = require("BuildFactory")
+BuildFactory:master()
+
 require("myGlobals")
 require("utils")
 require("colorize")
@@ -69,17 +98,17 @@ Cache               = require("Cache")
 MT                  = require("MT")
 Master              = require("Master")
 BaseShell           = require("BaseShell")
+local malias        = require("MAlias"):build()
 local lfs           = require("lfs")
 local json          = require("json")
 local dbg           = require("Dbg"):dbg()
 local Optiks        = require("Optiks")
 local Spider        = require("Spider")
 local concatTbl     = table.concat
-local posix         = require("posix")
 local sort          = table.sort
 
 
-local ignoreT     = {
+local ignoreA     = {
    "^$",
    "^%.$",
    "^%$",
@@ -94,22 +123,59 @@ local ignoreT     = {
    "^/opt/local/bin$",
 }
 
-function keepThisPath(path)
-   for i = 1, #ignoreT do
-      if (path:find(ignoreT[i])) then
+--      key:  /opt/apps/gcc-4_8/mpich/3.1.1,
+--      path: /opt/apps/gcc-4_8/mpich/3.1.1/lib, i: nil, j: nil
+
+
+
+--------------------------------------------------------------------------
+-- Check path against the *ignoreT* table.  Also it must be "in" dirA if
+-- dirA exists.
+-- @param path input path.
+local function keepThisPath2(path, dirA)
+   dbg.start{"keepThisPath2(",path,", dirA)"}
+
+   if (dirA) then
+      local match = false
+      for k in pairs(dirA) do
+         local i, j = path:find(k,1,true)
+         dbg.print{"key: \"",k, "\", path: \"",path,"\", i: ",i,", j: ",j,"\n"}
+         if ( i == 1) then
+            match = true
+            break
+         end
+      end
+      if (not match) then
+         dbg.print{"No match\n"}
+         dbg.fini("keepThisPath2")
          return false
       end
    end
+
+   for i = 1, #ignoreA do
+      if (path:find(ignoreA[i])) then
+         dbg.print{"In ignore list\n"}
+         dbg.fini("keepThisPath2")
+         return false
+      end
+   end
+   dbg.print{"Keep: true\n"}
+   dbg.fini("keepThisPath2")
    return true
 end
 
-local function add2map(entry, tbl, rmapT, kind)
+--------------------------------------------------------------------------
+-- Store an entry in *rmapT*
+-- @param entry
+-- @param tbl
+-- @param rmapT
+-- @param kind
+local function add2map(entry, tbl, dirA, moduleFn, rmapT, kind)
    for path in pairs(tbl) do
       local attr = lfs.attributes(path)
-      path = path_regularize(path)
-      if (keepThisPath(path) and attr and attr.mode == "directory") then
+      if (keepThisPath2(path,dirA) and attr and attr.mode == "directory") then
          path = abspath(path)
-         local t       = rmapT[path] or {pkg=entry.full, kind = kind, flavorT = {}}
+         local t       = rmapT[path] or {pkg=entry.full, kind = kind, moduleFn = moduleFn, flavorT = {}}
          local flavorT = t.flavorT
          for i = 1, #entry.parent do
             flavorT[entry.parent[i]] = true
@@ -119,6 +185,12 @@ local function add2map(entry, tbl, rmapT, kind)
    end
 end
 
+--------------------------------------------------------------------------
+--
+-- @param moduleDirA
+-- @param moduleT
+-- @param timestampFn
+-- @param dbT
 local function rptList(moduleDirA, moduleT, timestampFn, dbT)
    dbg.start{"rptList(moduleDirA, moduleT, timestampFn, dbT)"}
    local tbl = {}
@@ -133,10 +205,11 @@ end
 local function rptModuleT(moduleDirA, moduleT, timestampFn, dbT)
    dbg.start{"rptModuleT(moduleDirA, moduleT, timestampFn, dbT)"}
    local ts = { timestampFn }
+   local s0 = malias:export()
    local s1 = serializeTbl{name="timestampFn",   value=ts,         indent=true}
    local s2 = serializeTbl{name="defaultMpathA", value=moduleDirA, indent=true}
    local s3 = serializeTbl{name="moduleT",       value=moduleT,    indent=true}
-   io.stdout:write(s1,s2,s3,"\n")
+   io.stdout:write(s0,s1,s2,s3,"\n")
    dbg.fini("rptModuleT")
 end
 
@@ -146,10 +219,13 @@ local function buildReverseMapT(moduleDirA, moduleT, dbT)
    for kkk,vvv in pairs(dbT) do
       for kk, entry in pairs(vvv) do
          if (entry.pathA) then
-            add2map(entry, entry.pathA, reverseMapT,"bin")
+            add2map(entry, entry.pathA,  entry.dirA, kk, reverseMapT, "bin")
          end
          if (entry.lpathA) then
-            add2map(entry, entry.lpathA, reverseMapT,"lib")
+            add2map(entry, entry.lpathA, entry.dirA, kk, reverseMapT, "lib")
+         end
+         if (entry.dirA) then
+            add2map(entry, entry.dirA,   entry.dirA, kk, reverseMapT, "dir")
          end
       end
    end
@@ -167,28 +243,83 @@ local function buildReverseMapT(moduleDirA, moduleT, dbT)
    return reverseMapT
 end
 
+local function buildXALTrmapT(reverseMapT)
+   local rmapT = {}
+   for path,entry in pairs(reverseMapT) do
+      local value  = entry.pkg
+      local flavor = entry.flavor[1]
+      flavor       = flavor:gsub("default:?","")
+      if (flavor ~= "") then
+         value = value .. "(" .. flavor .. ")"
+      end
+      rmapT[path] = value
+   end
+   return rmapT
+end
+  
+local function buildLibMapA(reverseMapT)
+   local libT = {}
+   for path,v in pairs(reverseMapT) do
+      local kind = v.kind
+      if (kind == "lib") then
+         for file in lfs.dir(path) do
+            local ext = extname(file)
+            if (ext == ".a" or ext == ".so" or ext == ".dylib") then
+               libT[file] = true
+            end
+         end
+      end
+   end
+
+   local libA = {}
+   local i    = 0
+   for k in pairs(libT) do
+      i       = i + 1
+      libA[i] = k
+   end
+
+   return libA
+end
+
+
 
 local function rptReverseMapT(moduleDirA, moduleT, timestampFn, dbT)
    dbg.start{"rptReverseMapT(moduleDirA, moduleT, timestampFn, dbT)"}
    local ts          = { timestampFn }
    local reverseMapT = buildReverseMapT(moduleDirA, moduleT, dbT)
+   local libA        = buildLibMapA(reverseMapT)
    local s1          = serializeTbl{name="timestampFn",   value=ts,
                                     indent=true}
    local s2          = serializeTbl{name="reverseMapT",
                                     value=reverseMapT,   indent=true}
-   io.stdout:write(s1,s2,"\n")
+   local s3          = serializeTbl{name="xlibmap", value = libA,
+                                    indent = true}
+   io.stdout:write(s1,s2,s3,"\n")
    dbg.fini("rptReverseMapT")
 end
 
 local function rptReverseMapTJson(moduleDirA, moduleT, timestampFn, dbT)
    dbg.start{"rptReverseMapTJson(moduleDirA, moduleT, timestampFn, dbT)"}
    local reverseMapT = buildReverseMapT(moduleDirA, moduleT, dbT)
+   local libA        = buildLibMapA(reverseMapT)
    local t           = { timestampFn = timestampFn,
-                         reverseMapT = reverseMapT}
+                         reverseMapT = reverseMapT,
+                         xlibmap     = libA}
    print(json.encode(t))
    dbg.fini("rptReverseMapTJson")
 end
 
+local function rptXALTRmapTJson(moduleDirA, moduleT, timestampFn, dbT)
+   dbg.start{"rptXALTRmapTJson(moduleDirA, moduleT, timestampFn, dbT)"}
+   local reverseMapT = buildReverseMapT(moduleDirA, moduleT, dbT)
+   local libA        = buildLibMapA(reverseMapT)
+   local rmapT       = buildXALTrmapT(reverseMapT)
+   local t           = { reverseMapT = rmapT,
+                         xlibmap     = libA}
+   print(json.encode(t))
+   dbg.fini("rptXALTRmapTJson")
+end
+
 local function rptSoftwarePageJson(moduleDirA, moduleT, timestampFn, dbT)
    dbg.start{"rptSoftwarePageJson(moduleDirA, moduleT, timestampFn, dbT)"}
    local spA = softwarePage(dbT)
@@ -214,9 +345,10 @@ end
 local function rptDbT(moduleDirA, moduleT, timestampFn, dbT)
    dbg.start{"rptDbT(moduleDirA, moduleT, timestampFn, dbT)"}
    local ts = { timestampFn }
+   local s0 = malias:export()
    local s1 = serializeTbl{name="timestampFn",   value=ts,         indent=true}
    local s2 = serializeTbl{name="dbT",      value=dbT,   indent=true}
-   io.stdout:write(s1,s2,"\n")
+   io.stdout:write(s0,s1,s2,"\n")
    dbg.fini("rptDbT")
 end
 
@@ -237,6 +369,7 @@ function main()
 
    local master     = Master:master(false)
    master.shell     = BaseShell.build("bash")
+   parseVersion     = buildParseVersion()
 
    for _, v in ipairs(pargs) do
       for path in v:split(":") do
@@ -245,25 +378,23 @@ function main()
    end
 
 
-   if (masterTbl.debug) then
-      dbg:activateDebug(1)
+   if (masterTbl.debug > 0 or masterTbl.dbglvl) then
+      local dbgLevel = math.max(masterTbl.debug, masterTbl.dbglvl or 1)
+      dbg:activateDebug(dbgLevel)
    end
 
    dbg.start{"Spider main()"}
    MCP = MasterControl.build("spider")
    mcp = MasterControl.build("spider")
 
-   readRC()
-   build_epoch()            -- build the epoch function
-   build_accept_functions() -- functions to accept or ignore TCL mfiles
-   local cache = Cache:cache{dontWrite = true, quiet = true}
+   local cache = Cache:cache{dontWrite = true, quiet = true, buildCache = true}
 
    ------------------------------------------------------------------------
    -- do not colorize output from spider
    colorize = plain
 
    setenv_lmod_version() -- push Lmod version info into env for modulefiles.
-   
+
    ------------------------------------------------------------------------
    --  The StandardPackage is where Lmod registers hooks.  Sites may
    --  override the hook functions in SitePackage.
@@ -313,6 +444,8 @@ function main()
       reverseMap       = rptReverseMapT,
       jsonReverseMapT  = rptReverseMapTJson,
       jsonReverseMap   = rptReverseMapTJson,
+      xalt_rmapT       = rptXALTRmapTJson,
+      xalt_rmap        = rptXALTRmapTJson,
       ["spider-json"]  = rptDbTJson,
       dbT              = rptDbT,
    }
@@ -402,10 +535,17 @@ function options()
    local cmdlineParser = Optiks:new{usage=usage, version="1.0"}
 
    cmdlineParser:add_option{
-      name   = {'-D','--debug'},
+      name   = {'-D'},
       dest   = 'debug',
-      action = 'store_true',
-      default = false,
+      action = 'count',
+      help   = "Program tracing written to stderr",
+   }
+
+   cmdlineParser:add_option{
+      name   = {'--debug'},
+      dest   = 'dbglvl',
+      action = 'store',
+      help   = "Program tracing written to stderr",
    }
 
    cmdlineParser:add_option{
@@ -415,7 +555,7 @@ function options()
       default = "list",
       help    = "Output Style: list, moduleT, dbT, reverseMapT, "..
                 "jsonReverseMapT, spider, spider-json, softwarePage, "..
-                "jsonSoftwarePage, xmlSoftwarePage"
+                "jsonSoftwarePage, xmlSoftwarePage, xalt_rmapT"
    }
 
    cmdlineParser:add_option{
@@ -452,6 +592,7 @@ function options()
    Use_Preload = masterTbl.preload
 end
 
+xml = false
 function xmlSoftwarePage(dbT)
    require("LuaXml")  -- This defines xml
 
@@ -521,7 +662,7 @@ function findLatestV(a)
       aa[i] = { concatTbl(b,":"), entry}
    end
 
-   table.sort(aa, function(a,b) return a[1] > b[1] end)
+   table.sort(aa, function(x,y) return x[1] > y[1] end)
 
    local result = aa[1][2]
    if (result ~= "default") then
@@ -537,6 +678,7 @@ function localSoftware(xml, name, t)
 
    local root = xml.new("LocalSoftware")
 
+   local Name
    local value  = "unknown"
    local domain = "unknown"
    local category = t.Category or ""
@@ -562,18 +704,18 @@ function localSoftware(xml, name, t)
       value = "application"
    end
 
-   local Name = xml.new("Type")
-   Name[1]    = value
+   Name    = xml.new("Type")
+   Name[1] = value
    root:append(Name)
    dbg.print{"Type: ",value,"\n"}
 
-   local Name = xml.new("Domain")
-   Name[1]    = domain
+   Name    = xml.new("Domain")
+   Name[1] = domain
    root:append(Name)
    dbg.print{"domain: ",domain,"\n"}
 
-   local Name = xml.new("Name")
-   Name[1]    = name
+   Name    = xml.new("Name")
+   Name[1] = name
    root:append(Name)
    dbg.print{"name: ",name,"\n"}
 
