@@ -42,28 +42,32 @@ require("strict")
 require("fileOps")
 require("string_utils")
 require("parseVersion")
+require("myGlobals")
 require("capture")
 
+_G._DEBUG          = false               -- Required by the new lua posix
 local Version      = require("Version")
 local base64       = require("base64")
 local dbg          = require("Dbg"):dbg()
 local lfs          = require("lfs")
+local malias       = require("MAlias"):build()
 local posix        = require("posix")
+local readlink     = posix.readlink
 local setenv_posix = posix.setenv
-
 local concatTbl    = table.concat
 local decode64     = base64.decode64
 local floor        = math.floor
 local format       = string.format
 local getenv       = os.getenv
 local huge         = math.huge
+local open         = io.open
 
 local rep          = string.rep
 local load         = (_VERSION == "Lua 5.1") and loadstring or load
 
 --------------------------------------------------------------------------
 -- find true path through symlinks.
-
+-- @param path A file path
 function abspath_localdir (path)
    if (path == nil) then return nil end
    local cwd = lfs.currentdir()
@@ -115,72 +119,87 @@ end
 --------------------------------------------------------------------------
 -- This is 5.1 Lua function to cover the table.pack function
 -- that is in Lua 5.2 and later.
-
 function argsPack(...)
-   local arg = { n = select ("#", ...), ...}
+   local arg = { n = select("#", ...), ...}
    return arg
 end
-local pack        = (_VERSION == "Lua 5.1") and argsPack or table.pack
+pack     = (_VERSION == "Lua 5.1") and argsPack or table.pack
 
 
 --------------------------------------------------------------------------
--- This function builds the "epoch" function.  
--- The epoch function returns the number of seconds since
--- Jan 1, 1970, UTC
+-- Generate a message that will fix the available terminal width.
+-- @param width The terminal width
+function buildMsg(width, ... )
+   local arg = pack(...)
+   local a   = {}
+   local len = 0
 
-function build_epoch()
-   if (posix.gettimeofday) then
-      local x1, x2 = posix.gettimeofday()
-      if (x2 == nil) then
-         epoch_type = "posix.gettimeofday() (1)"
-         epoch = function()
-            local t = posix.gettimeofday()
-            return t.sec + t.usec*1.0e-6
+   for idx = 1, arg.n do
+      local block  = arg[idx]
+      local done   = false
+      while (not done) do
+         local hasNL  = false
+         local ja, jb = block:find("\n")
+         local line
+         if (ja) then
+            hasNL = true
+            line  = block:sub(1,ja-1)
+            block = block:sub(ja+1,-1)
+         else
+            done  = true
+            line  = block
          end
-      else
-         epoch_type = "posix.gettimeofday() (2)"
-         epoch = function()
-            local t1, t2 = posix.gettimeofday()
-            return t1 + t2*1.0e-6
+
+         if (#a > 0 and line:sub(1,1) == '"' and a[#a] == " ") then
+            a[#a] = nil
+         end
+
+         local i,j, lBlnks = line:find("^( +)")
+         local rest        = line
+         if (i) then
+            a[#a+1] = lBlnks
+            len     = len + lBlnks:len()
+            rest    = line:sub(j+1,-1)
+         end
+
+         for word in rest:split(" +") do
+            local wlen = word:len()
+            if (wlen + len >= width) then
+               a[#a]     = "\n"
+               len       = 0
+            end
+            if (wlen > 0) then
+               a[#a + 1] = word
+               a[#a + 1] = " "
+            end
+            len = len + wlen + 1
+         end
+         if (hasNL) then
+            local jc = #a+1
+            if (a[#a] == " ") then
+               jc = #a
+            end
+            a[jc] = "\n"
+            len   = 0
+         end
+         if (a[#a] == " ") then
+            a[#a] = nil
+         end
+         local last = a[#a]:sub(-1)
+         if (last ~= '"' and last ~= "\n") then
+            a[#a + 1] = " "
          end
       end
-   else
-      epoch_type = "os.time"
-      epoch = function()
-         return os.time()
-      end
    end
+
+   return concatTbl(a,"")
 end
 
---------------------------------------------------------------------------
--- Create the accept functions to allow or ignore TCL files.
-
-function build_accept_functions()
-   local allow_tcl = LMOD_ALLOW_TCL_MFILES:lower()
-
-   if (allow_tcl == "no") then
-      dbg.print{"Ignoring TCL Files\n"}
-      _G.accept_fn = function (fn)
-         return fn:find("%.lua$")
-      end
-      _G.accept_extT = function ()
-         return { '.lua' }
-      end
-
-   else
-      dbg.print{"Accepting TCL Files\n"}
-      _G.accept_fn = function (fn)
-         return true
-      end
-      _G.accept_extT = function ()
-         return { '.lua', '' }
-      end
-   end
-end
 
 --------------------------------------------------------------------------
--- What it says.
-
+-- Convert both argument to lower case and compare.
+-- @param a input string
+-- @param b input string
 function case_independent_cmp(a,b)
    local a_lower = a:lower()
    local b_lower = b:lower()
@@ -193,9 +212,9 @@ function case_independent_cmp(a,b)
 end
 
 
---------------------------------------------------------------------------
--- Are we in expert mode?
 local __expert = false
+--------------------------------------------------------------------------
+-- Return true if in expert mode.
 function expert()
    if (__expert == false) then
       __expert = getenv("LMOD_EXPERT")
@@ -203,9 +222,9 @@ function expert()
    return __expert
 end
 
---------------------------------------------------------------------------
--- Are we in quiet mode?
 local __quiet = false
+--------------------------------------------------------------------------
+-- Return ture if in quiet mode.
 function quiet()
    if (__quiet == false) then
       __quiet = getenv("LMOD_QUIET") or getenv("LMOD_EXPERT")
@@ -213,11 +232,26 @@ function quiet()
    return __quiet
 end
 
+function isActiveMFile(full, sn)
+   local version = extractVersion(full, sn) or ""
+   if (version:sub(1,1) == ".") then
+      return false, version
+   end
+   if (full:sub(1,1) == ".") then
+      return false, version
+   end
+   if (full:find("/%.")) then
+      return false, version
+   end
+   return true, version
+end
+
 --------------------------------------------------------------------------
 -- Compare the full name of a modulefile with the
 -- shortname. Return nil if the shortname and full name
 -- are the same.
-
+-- @param full full module name
+-- @param sn   short name
 function extractVersion(full, sn)
    if (not full or not sn) then
       return nil
@@ -229,15 +263,12 @@ function extractVersion(full, sn)
       return nil
    end
 
-
-   full = full:sub(1,j):lower() .. full:sub(j+1,-1)
-   sn   = sn:lower()
-
    local pat     = '^' .. sn:escape() .. '/?'
    local version = full:gsub(pat,"")
    if (version == "") then
       version = nil
    end
+
    return version
 end
 
@@ -246,6 +277,7 @@ end
 s_ignoreT = {
    ['.']         = true,
    ['..']        = true,
+   ['.lua']      = true,
    ['.moduler']  = true,
    ['.version']  = true,
    ['.modulerc'] = true,
@@ -255,7 +287,8 @@ s_ignoreT = {
    --@ignore_dirs@--
 }
 
-
+--------------------------------------------------------------------------
+-- Return the table of files to ignore when searching for modulefiles.
 function ignoreFileT()
    local fileT = s_ignoreT
    return fileT
@@ -269,7 +302,6 @@ end
 -- quotes and parens won't confuse the shell's poor little
 -- brain.  The number of pieces are stored in the global
 -- env. variable _ModuleTable_Sz_.
-
 function getMT()
    local a    = {}
    local mtSz = getenv("_ModuleTable_Sz_") or huge
@@ -289,6 +321,10 @@ function getMT()
    return s
 end
 
+--------------------------------------------------------------------------
+-- Find all the versions in the pathA or at least *n* of them.
+-- @param pathA  an array of paths.
+-- @param n      a number of paths to check.
 function allVersions(pathA, n)
    local lastKey   = ''
    local lastValue = ''
@@ -296,30 +332,24 @@ function allVersions(pathA, n)
    local fullName  = nil
    local count     = 0
    local a         = {}
-   local n         = n or #pathA
-   local accept_fn = accept_fn
+   n               = n or #pathA
 
    for i = 1, n do
-      local vv   = pathA[i]
-      local path = vv.file
-      local attr = lfs.attributes(path)
-      if (attr and attr.mode == 'directory' and posix.access(path,"x")) then
-         for v in lfs.dir(path) do
-            local f = pathJoin(path, v)
-            attr    = lfs.attributes(f)
-            local readable = posix.access(f,"r") and accept_fn(f)
-            if (readable and v:sub(1,1) ~= "." and attr.mode == 'file'
-                and v:sub(-1,-1) ~= '~') then
-               v       = v:gsub("%.lua$","")
-               local pv = parseVersion(v)
-               a[#a+1] = {version=v, file=f, idx = i, mpath=vv.mpath, pv=pv}
-            end
-         end
+      local vv = pathA[i]
+      for version, fn in pairs(vv.versionT) do
+         local pv = parseVersion(version)
+         a[#a + 1] = {version = version, file = fn, idx = i, mpath = vv.mpath, pv = pv}
       end
    end
    return a
 end
 
+--------------------------------------------------------------------------
+-- Compare two path arrays.  Return the index where they start to differ.
+-- @param old   Old path as a string
+-- @param oldA  An array of old path entries
+-- @param new   New path as a string
+-- @param newA  An array of new path entries
 function indexPath(old, oldA, new, newA)
    dbg.start{"indexPath(",old, ", ", new,")"}
    local oldN = #oldA
@@ -382,8 +412,8 @@ end
 -- function to decide which version is the most recent.
 -- It is not a lexigraphical search but uses rules built
 -- into parseVersion().
-
-
+-- @param pathA  an array of paths.
+-- @param n      a number of paths to check.
 function lastFileInPathA(pathA, n)
    local lastKey   = ''
    local lastValue = ''
@@ -408,17 +438,16 @@ end
 --------------------------------------------------------------------------
 -- compute the length of a string and ignore any string
 -- colorization.
-
+-- @param s input string
 function length(s)
    s = s:gsub("\027[^m]+m","")
    return s:len()
 end
 
+s_masterTbl = {}
 --------------------------------------------------------------------------
 -- Manage the Master Hash Table.  The command line arguments
 -- and the ModuleStack (when using Spider) are stored here.
-
-s_masterTbl = {}
 function masterTbl()
    return s_masterTbl
 end
@@ -437,7 +466,9 @@ end
 -- To handle that, the path component is converted to
 -- a single space.  This single space is later removed
 -- when expanding.
-
+-- @param path A string of *sep* separated paths.
+-- @param sep  The separator character.  It is usually
+--             a colon.
 function path2pathA(path, sep)
    sep = sep or ":"
    if (not path) then
@@ -449,17 +480,6 @@ function path2pathA(path, sep)
 
    local is, ie
 
-   -- remove leading and trailing sep
-   if (path:sub(1,1) == sep) then
-      is = 2
-   end
-   if (path:sub(-1,-1) == sep) then
-      ie = -2
-   end
-   if (is) then
-      path = path:sub(is,ie)
-   end
-
    local pathA = {}
    for v  in path:split(sep) do
       pathA[#pathA + 1] = path_regularize(v)
@@ -479,21 +499,10 @@ function path2pathA(path, sep)
 end
 
 --------------------------------------------------------------------------
---  Read the admin.list file.  It is a Key value pairing of module names
---  and a message.  The module names can be either the full name or the file
---  name.  The message can be multi-line.  A blank line is signifies the
---  end of the message.
---
---  /path/to/modulefile: Blah Blah Blah
---                       Blah Blah Blah
---
---  module/version:      Blah Blah Blah
---                       Blah Blah Blah
---
-
+-- Find the admin file (or nag message file).
 function findAdminFn()
    local readable    = "no"
-   local adminFn     = getenv("LMOD_ADMIN_FILE") or pathJoin(cmdDir(),"../../etc/admin.list")
+   local adminFn     = LMOD_ADMIN_FILE
    local dirName, fn = splitFileName(adminFn)
    if (isDir(dirName)) then
       local cwd      = posix.getcwd()
@@ -505,11 +514,22 @@ function findAdminFn()
          readable = "yes"
       end
    end
-   
+
    return adminFn, readable
 end
 
 
+--------------------------------------------------------------------------
+--  Read the admin.list file.  It is a Key value pairing of module names
+--  and a message.  The module names can be either the full name or the file
+--  name.  The message can be multi-line.  A blank line is signifies the
+--  end of the message.
+--
+--  /path/to/modulefile: Blah Blah Blah
+--                       Blah Blah Blah
+--
+--  module/version:      Blah Blah Blah
+--                       Blah Blah Blah
 function readAdmin()
 
    -- If there is anything in [[adminT]] then return because
@@ -517,11 +537,11 @@ function readAdmin()
    if (next (adminT)) then return end
 
    local adminFn = findAdminFn()
-   local f       = io.open(adminFn,"r")
+   local f       = open(adminFn,"r")
 
    -- Put something in adminT so that this routine will not be
    -- run again even if the file does not exist.
-   adminT["foo"] = "bar"
+   adminT["%%_foo_%%"] = "bar"
 
    if (f) then
       local whole = f:read("*all") .. "\n"
@@ -536,94 +556,108 @@ function readAdmin()
       local a     = {}
 
       for v in whole:split("\n") do
+         repeat
+            v = v:trim()
 
-         v = v:trim()
+            if (v:sub(1,1) == "#") then
+               -- ignore this comment line
+               break
 
-         if (v:sub(1,1) == "#") then
-            -- ignore this comment line
+            elseif (v:find("^%s*$")) then
+               if (state == "value") then
+                  value       = concatTbl(a, " ")
+                  a           = {}
+                  adminT[key] = value
+                  state       = "init"
+               end
 
-
-         elseif (v:find("^%s*$")) then
-            if (state == "value") then
-               value       = concatTbl(a, " ")
-               a           = {}
-               adminT[key] = value
-               state       = "init"
-            end
-
-            -- Ignore blank lines
-         elseif (state == "value") then
-            a[#a+1]     = v:trim()
-         else
-            local i     = v:find(":")
-            if (i) then
-               key      = v:sub(1,i-1):trim()
-               local  s = v:sub(i+1):trim()
-               if (s:len() > 0) then
-                  a[#a+1]  = s
+               -- Ignore blank lines
+            elseif (state == "value") then
+               a[#a+1]     = v:trim()
+            else
+               local i     = v:find(":")
+               if (i) then
+                  key      = v:sub(1,i-1):trim()
+                  local  s = v:sub(i+1):trim()
+                  if (s:len() > 0) then
+                     a[#a+1]  = s
+                  end
+                  state    = "value"
                end
-               state    = "value"
             end
-         end
+         until true
       end
    end
 end
 
+--local s_readRC     = false
+--RCFileA = {
+--   pathJoin(cmdDir(),"../init/lmodrc.lua"),
+--   pathJoin(cmdDir(),"../../etc/lmodrc.lua"),
+--   pathJoin("/etc/lmodrc.lua"),
+--   pathJoin(getenv("HOME"),".lmodrc.lua"),
+--   os.getenv("LMOD_RC"),
+--}
+--
+----------------------------------------------------------------------------
+---- Read in the system and possible a user lmod configuration file.
+---- The system one is read first.  These provide default value
+---- The user one can override the default values.
+--function readRC()
+--   dbg.start{"readRC()"}
+--   if (s_readRC) then
+--      s_readRC = true
+--      return
+--   end
+--
+--   declare("propT",       false)
+--   declare("scDescriptT", false)
+--
+--   for i = 1,#RCFileA do
+--      repeat
+--         local f        = RCFileA[i]
+--         local fh = open(f)
+--         if (not fh) then break end
+--            
+--         assert(loadfile(f))()
+--         s_rcFileA[#s_rcFileA+1] = abspath(f)
+--         fh:close()
+--
+--         local propT       = _G.propT or {}
+--         local scDescriptT = _G.scDescriptT   or {}
+--         for k,v in pairs(propT) do
+--            s_propT[k] = v
+--         end
+--         for j = 1,#scDescriptT do
+--            s_scDescriptT[#s_scDescriptT + 1] = scDescriptT[j]
+--         end
+--      until true
+--   end
+--   dbg.fini("readRC")
+--end
+--
+----------------------------------------------------------------------------
+---- Return the property table.
+--function getPropT()
+--   return s_propT
+--end
+--
+----------------------------------------------------------------------------
+---- Return the spider cache description table.
+--function getSCDescriptT()
+--   return s_scDescriptT
+--end
+--
+--
+----------------------------------------------------------------------------
+---- Return the array of active RC files
+--function getRCFileA()
+--   return s_rcFileA
+--end
+
 --------------------------------------------------------------------------
--- read in the system and possible a user lmod configuration file.
--- The system one is read first.  These provide default value
--- The user one can override the default values.
-
-local s_readRC     = false
-RCFileA = {
-   pathJoin(cmdDir(),"../init/lmodrc.lua"),
-   pathJoin(cmdDir(),"../../etc/lmodrc.lua"),
-   pathJoin(getenv("HOME"),".lmodrc.lua"),
-   os.getenv("LMOD_RC"),
-}
-
-function readRC()
-   dbg.start{"readRC()"}
-   if (s_readRC) then
-      s_readRC = true
-      return
-   end
-
-   declare("propT",       false)
-   declare("scDescriptT", false)
-
-   for i = 1,#RCFileA do
-      local f        = RCFileA[i]
-      local fh = io.open(f)
-      if (fh) then
-         assert(loadfile(f))()
-         s_rcFileA[#s_rcFileA+1] = abspath(f)
-         fh:close()
-      end
-      local propT       = _G.propT or {}
-      local scDescriptT = _G.scDescriptT   or {}
-      for k,v in pairs(propT) do
-         s_propT[k] = v
-      end
-      for k,v in pairs(scDescriptT) do
-         s_scDescriptT[k] = v
-      end
-   end
-   dbg.fini("readRC")
-end
-
-function getPropT()
-   return s_propT
-end
-
-function getSCDescriptT()
-   return s_scDescriptT
-end
-
-function getRCFileA()
-   return s_rcFileA
-end
-
+-- Convert number and string to a quoted string.
+-- @param v input number or string.
 local function arg2str(v)
    if (v == nil) then return v end
    local s = tostring(v)
@@ -642,7 +676,9 @@ defaultsT = {
    priority = "0",
 }
 
-
+--------------------------------------------------------------------------
+-- This routine converts a command into a string.  This is used by MC_Show
+-- @param name Input command name.
 function ShowCmdStr(name, ...)
    local a       = {}
    local arg     = pack(...)
@@ -685,10 +721,15 @@ function ShowCmdStr(name, ...)
    b[#b+1] = name
    b[#b+1] = left
    b[#b+1] = concatTbl(a,",")
-   b[#b+1] = right 
+   b[#b+1] = right
    return concatTbl(b,"")
 end
 
+
+--------------------------------------------------------------------------
+-- This routine converts a command into a string.  This is used by MC_Show
+-- @param name Input command name.
+-- @param mA   An array of Module Name objects.
 function ShowCmdA(name, mA)
    local a = {}
    for i = 1, #mA do
@@ -707,7 +748,6 @@ end
 --------------------------------------------------------------------------
 -- Unique string that combines the current time/date
 -- with a uuid id string.
-
 function UUIDString(epoch)
    local ymd  = os.date("*t", epoch)
 
@@ -716,59 +756,51 @@ function UUIDString(epoch)
                                    ymd.year, ymd.month, ymd.day,
                                    ymd.hour, ymd.min,   ymd.sec)
 
-   local uuid_str  = capture("uuidgen"):sub(1,-2)
+   local uuidgen = find_exec_path('uuidgen')
+   local uuid_str
+   if uuidgen then
+       uuid_str = capture('uuidgen'):sub(1,-2)
+   else
+      -- if uuidgen is not available, fall back to reading /proc/sys/kernel/random/uuid
+      -- note: only works on Linux
+      local f = open('/proc/sys/kernel/random/uuid', 'r')
+      if f then
+         uuid_str = f:read('*all'):sub(1,-2)
+      else
+         LmodError("uuidgen is not available, fallback failed too")
+      end
+   end
+
    local uuid      = uuid_date .. "-" .. uuid_str
 
    return uuid
 end
-modV = false
 
-function moduleRCFile(current, path)
-   dbg.start{"moduleRCFile(",path,")"}
-   local f       = io.open(path,"r")
-   if (not f)                        then
-      dbg.print{"could not find: ",path,"\n"}
-      dbg.fini("moduleRCFile")
-      return nil
-   end
-   local s       = f:read("*line")
-   f:close()
-   if (not s:find("^#%%Module"))      then
-      dbg.print{"could not find: #%Module\n"}
-      dbg.fini("moduleRCFile")
-      return nil
-   end
-   local cmd = pathJoin(cmdDir(),"RC2lua.tcl") .. " " .. path
-   local s = capture(cmd):trim()
-   assert(load(s))()
-   local version = false
-   for i = 1,#modV do
-      local entry = modV[i]
-      if (entry.module_version == "default") then
-         local name = entry.module_name
-         local i, j = name:find(current)
-         local nLen = name:len()
-         if (j+1 < nLen and name:sub(j+1,j+1) == '/') then
-            version = name:sub(j+2)
-            break
-         end
-      end
-   end
+modA = false
 
-   dbg.print{"version: ",version,"\n"}
-   dbg.fini("moduleRCFile")
-   return version
-   
+function runTCLprog(TCLprog, optStr, fn)
+   local a   = {}
+   a[#a + 1] = LMOD_TCLSH
+   a[#a + 1] = pathJoin(cmdDir(),TCLprog)
+   a[#a + 1] = optStr or ""
+   a[#a + 1] = fn
+   local cmd = concatTbl(a," ")
+   local whole, status = capture(cmd)
+   return whole, status
 end
+   
 --------------------------------------------------------------------------
--- This routine is given the absolute path to a .version 
+-- This routine is given the absolute path to a .version
 -- file.  It checks to make sure that it is a valid TCL
--- file.  It then uses the ModulesVersion.tcl script to 
+-- file.  It then uses the ModulesVersion.tcl script to
 -- return what the value of "ModulesVersion" is.
-
+-- @param v The version file name: {.modulerc, .version}
+-- @param sn The short name
+-- @param path The path to version file
+-- @param ignoreErrors If true then ignore errors.
 function versionFile(v, sn, path, ignoreErrors)
    dbg.start{"versionFile(v: ",v,", sn: ",sn,", path: ",path,")"}
-   local f       = io.open(path,"r")
+   local f       = open(path,"r")
    if (not f)                        then
       dbg.print{"could not find: ",path,"\n"}
       dbg.fini("versionFile")
@@ -782,54 +814,24 @@ function versionFile(v, sn, path, ignoreErrors)
       return nil
    end
    local version = false
-
-   if (v == "/.modulerc") then
-      local cmd = pathJoin(cmdDir(),"RC2lua.tcl") .. " " .. path
-      local s = capture(cmd):trim()
-      assert(load(s))()
-      for i = 1,#modV do
-         local entry = modV[i]
-         if (entry.module_version == "default") then
-            local name = entry.module_name
-            local i, j = name:find(sn)
-            local nLen = name:len()
-            if (j+1 < nLen and name:sub(j+1,j+1) == '/') then
-               version = name:sub(j+2)
-               break
-            end
-         end
-      end
-   elseif (v == "/.version") then
-      local cmd = pathJoin(cmdDir(),"ModulesVersion.tcl") .. " " .. path
-      local s = capture(cmd):trim()
-      assert(load(s))()
-      version = modV.version
-      if (modV.date ~= "***") then
-         local a = {}
-         for s in modV.date:split("/") do
-            a[#a + 1] = tonumber(s) or 0
-         end
-         
-         if (not ignoreErrors and (a[1] < 2000 or a[2] > 12 )) then
-            LmodMessage("The .version file for \"",sn,
-                        "\" has the date is written in the wrong format: \"",
-                        modV.date,"\".  Please use YYYY/MM/DD.")
-         end
-         
-         local epoch   = os.time{year = a[1], month = a[2], day = a[3]} or 0
-         local current = os.time() 
-         if (current < epoch) then
-            if (not ignoreErrors) then
-               LmodMessage("The default version for module \"",myModuleName(),
-                           "\" is changing on ", modV.date, " from ",modV.version,
-                           " to ", modV.newVersion,"\n")
-            end
-            version = modV.version
-         else
-            version = modV.newVersion
-         end
-      end
+   dbg.print{"handle file: ",v, "\n"}
+   local whole
+   local status
+   local func
+   whole, status = runTCLprog("RC2lua.tcl", "", path)
+   if (not status) then
+      LmodError("Unable to parse: ",path," Aborting!\n")
    end
+
+   status, func = pcall(load, whole)
+   if (not status or not func) then
+      LmodError("Unable to parse: ",path," Aborting!\n")
+   end
+   func()
+   malias:parseModA(sn, modA)
+
+   version = malias:getDefaultT(sn) or version
+
    dbg.print{"version: ",version,"\n"}
    dbg.fini("versionFile")
    return version
@@ -857,28 +859,68 @@ function setenv_lmod_version()
    end
 end
 
-
---------------------------------------------------------------------------
--- walk_directory_for_mf:
---     Walk a single directory for modulefiles and defaults:
---     Inputs:
---         mpath:
---         path:
---         prefix:
---     Outputs:
---         dirA:
---         mnameT:
---     Returns:
---         defaultFn:
-
 local defaultFnT = {
    default       = 1,
    ['.modulerc'] = 2,
    ['.version']  = 3,
 }
 
+--------------------------------------------------------------------------
+-- Find exceptable files. Or mark as false files that should be ignored
+-- @param fn input file name
+
+local ignoreT   = ignoreFileT()
+
+--------------------------------------------------------------------------
+-- Find exceptable files. Or mark as false files that should be ignored
+-- @param fn input file name
+function keepFile(fn)
+   local fileDflt  = fn:sub(1,8)
+   local firstChar = fn:sub(1,1)
+   local lastChar  = fn:sub(-1,-1)
+   local firstTwo  = fn:sub(1,2)
+
+   local result    = not (ignoreT[fn]      or lastChar == '~' or ignoreT[fileDflt] or
+                          firstChar == '#' or lastChar == '#' or firstTwo == '.#')
+   if (not result) then
+      return result
+   end
+
+   if (firstChar == "." and fn:sub(-4,-1) == ".swp") then
+      return false
+   end
+
+   return result
+end
+
+local function checkValidModulefileReal(fn)
+   local f = open(fn,"r")
+   if (not f) then
+      return false
+   end
+   local line = f:read(20) or ""
+   f:close()
+   return line:find("^#%%Module")
+end
+
+function checkValidModulefileFake(fn)
+   return true
+end
+
+local checkValidModulefile = (LMOD_CHECK_FOR_VALID_MODULE_FILES == "yes")
+                             and checkValidModulefileReal or checkValidModulefileFake
+
+
+--------------------------------------------------------------------------
+-- Walk a single directory for modulefiles and defaults:
+-- @param mpath Input modulepath directory
+-- @param path Input of the current directory.
+-- @param prefix the prefix
+-- @param dirA An array of directories found.
+-- @param mnameT A table of module names found
+-- @return defaultFn the default modulefile.
 function walk_directory_for_mf(mpath, path, prefix, dirA, mnameT)
-   dbg.start{"walk_directory_for_mf(",mpath,", ",path,", ",prefix,", dirA, mnameT)"}
+   dbg.start{"walk_directory_for_mf(mpath: ",mpath,", path: ",path,", prefix: \"",prefix,"\", dirA, mnameT)"}
    local attr = lfs.attributes(path)
    if (not attr or type(attr) ~= "table" or attr.mode ~= "directory"
        or not posix.access(path,"x")) then
@@ -893,46 +935,82 @@ function walk_directory_for_mf(mpath, path, prefix, dirA, mnameT)
    -----------------------------------------------------------------------------
    -- Read every relevant file in a directory.  Copy directory names into dirA.
    -- Copy files into mnameT.
-   local ignoreT   = ignoreFileT()
+   ignoreT   = ignoreFileT()
+
+   --local archNameT = {}
+   --local archNameT = { ['64'] = true, ['32'] = true, ['x86_64'] = true, ['ia32'] = true, gcc = true,
+   --                    ['haswell'] = true, ['ivybridge'] = true, ['sandybridge'] = true, ['ia32'] = true,
+   --}
+
+   local a = {}
+
 
    for file in lfs.dir(path) do
-      local idx       = defaultFnT[file] or defaultIdx
-      if (idx < defaultIdx) then
-         defaultIdx = idx
-         defaultFn  = pathJoin(path,file)
-      else
-         local fileDflt  = file:sub(1,8)
-         local firstChar = file:sub(1,1)
-         local lastChar  = file:sub(-1,-1)
-         local firstTwo  = file:sub(1,2)
-         
-         if (not (ignoreT[file]    or lastChar == '~' or ignoreT[fileDflt] or
-                  firstChar == '#' or lastChar == '#' or firstTwo == '.#')) then
-            local f        = pathJoin(path,file)
-            attr           = lfs.attributes(f) or {}
-            local readable = posix.access(f,"r")
-            local full     = pathJoin(prefix, file):gsub("%.lua","")
-            
-            ------------------------------------------------------------
-            -- Since cache files are build by root but read by users
-            -- make sure that any user can read a file owned by root.
+      repeat
+         local idx       = defaultFnT[file] or defaultIdx
+         if (idx < defaultIdx) then
+            defaultIdx = idx
+            defaultFn  = pathJoin(path,file)
+         else
+            if (keepFile(file))then
+               local f        = pathJoin(path,file)
+               attr           = lfs.attributes(f)
+               local readable = posix.access(f,"r")
+               local full     = pathJoin(prefix, file):gsub("%.lua","")
 
-            if (readable) then
-               local st    = posix.stat(f)
-               if (st.uid == 0 and not st.mode:find("......r..")) then
-                  readable = false
+               ------------------------------------------------------------
+               -- Since cache files are build by root but read by users
+               -- make sure that any user can read a file owned by root.
+
+               if (readable) then
+                  local st    = posix.stat(f)
+                  if ((not st) or (st.uid == 0 and not st.mode:find("......r.."))) then
+                     readable = false
+                  end
+               end
+
+               if (not readable or not attr) then
+                  -- do nothing for non-readable or non-existant files
+                  break
+               elseif (attr.mode == 'file' and file ~= "default" and accept_fn(file) and
+                       full:sub(1,1) ~= '.') then
+                  -- Lua modulefiles should always be picked over TCL modulefiles
+                  if (not mnameT[full] or not mnameT[full].luaExt) then
+                     local luaExt  = f:find("%.lua$")
+                     if (luaExt  or checkValidModulefile(f)) then
+                        local sn      = prefix:gsub("/$","")
+                        local version = file:gsub("%.lua$","")
+                        if (sn == "") then
+                           sn = full
+                           version = false
+                        end
+                        mnameT[full] = {fn = f, canonical=f:gsub("%.lua$",""), mpath = mpath,
+                                        luaExt = luaExt, version=version, sn=sn}
+                     end
+                  end
+               elseif (attr.mode == "directory" and file:sub(1,1) ~= ".") then
+                  -- Remember all directories in local array.  We have to know if
+                  -- any are "arch" directories.
+                  a[#a + 1] = { fullName = f, mname = full, file=file, path=path}
                end
             end
-            
-            if (not readable or not attr) then
-               -- do nothing for non-readable or non-existant files
-            elseif (attr.mode == 'file' and file ~= "default" and accept_fn(file) and
-                    full:sub(1,1) ~= '.') then
-               mnameT[full] = {fn = f, canonical=f:gsub("%.lua$",""), mpath = mpath}
-            elseif (attr.mode == "directory" and file:sub(1,1) ~= ".") then
-               dirA[#dirA + 1] = { fullName = f, mname = full}
-            end
          end
+      until true
+   end
+
+
+   for i = 1,#a do
+      dirA[#dirA + 1] = a[i]
+   end
+
+   if (dbg.active()) then
+      for i = 1,#dirA do
+         dbg.print{"dirA[",i,"].mname: ", dirA[i].mname,", \tdirA[",i,"].fullName: ",dirA[i].fullName,
+                   ", file: ",dirA[i].file,"\n"}
+      end
+      dbg.print{"\n"}
+      for k,v in pairsByKeys(mnameT) do
+         dbg.print{"mnameT[",k,"].fn: ",v.fn," sn: ",v.sn,", version: ", v.version,"\n"}
       end
    end
 
@@ -941,28 +1019,57 @@ function walk_directory_for_mf(mpath, path, prefix, dirA, mnameT)
 end
 
 --------------------------------------------------------------------------
--- Deal with warnings
+-- Use readlink to find the link
+-- @param path the path to the module file.
+function walk_link(path)
+   local attr   = lfs.symlinkattributes(path)
+   if (attr == nil) then
+      return nil
+   end
 
+   if (attr.mode == "link") then
+      local rl = readlink(path)
+      if (not rl) then
+         return nil
+      end
+      return pathJoin(dirname(path),rl)
+   end
+   return path
+end
+
+
+--------------------------------------------------------------------------
+-- Allow for the generation of warnings.
 function activateWarning()
    s_haveWarnings = true
 end
 
+--------------------------------------------------------------------------
+-- Disallow the generation of warnings.
 function deactivateWarning()
    s_haveWarnings = false
 end
 
+--------------------------------------------------------------------------
+-- Are the generation of warnings allowed?
 function haveWarnings()
    return s_haveWarnings
 end
 
+--------------------------------------------------------------------------
+-- Reset warning flag to false.
 function clearWarningFlag()
    s_warning = false
 end
 
+--------------------------------------------------------------------------
+-- Set warning flags to true.
 function setWarningFlag()
    s_warning = true
 end
 
+--------------------------------------------------------------------------
+-- Get warning flag value.
 function getWarningFlag()
    return s_warning
 end
