@@ -15,7 +15,7 @@
 --    5.  Compute actual number columns that will fit.
 --    6.  Knowing the number of rows and columns, build the table.
 --
--- @classmod ColumnTable 
+-- @classmod ColumnTable
 
 require("strict")
 
@@ -58,17 +58,6 @@ M = { gap = 3, innerGap = 1 }
 
 local blank = ' '
 
---------------------------------------------------------------------------
--- A recursive function to return the number of entries the current
--- dimension
--- @param a input array.
--- @param dim size of a in each dimension.
-local function sizeMeHelper(a,dim)
-   if (type (a) == "table") then
-      dim[#dim+1] = #a
-      sizeMeHelper(a[1],dim)
-   end
-end
 
 --------------------------------------------------------------------------
 -- compute the number of dimension in input table.
@@ -76,7 +65,15 @@ end
 -- @return an array with the number of entries of a in each dimension
 local function sizeMe(a)
    local dim = {}
-   sizeMeHelper(a,dim)
+   local rows = #a
+   dim[1] = rows
+   if (type(a[1]) == "table") then
+      local cols = 0
+      for i = 1,#a do
+         cols = max(cols,#a[i])
+      end
+      dim[2] = cols
+   end
    return dim
 end
 
@@ -85,8 +82,9 @@ end
 -- @param self ColumnTable object
 -- @param t input table.
 function M.new(self,t)
-   local tbl = t
-   local o   = {}
+   local width
+   local tbl   = t
+   local o     = {}
    if (t.tbl) then
       tbl = t.tbl
       o   = t
@@ -95,8 +93,9 @@ function M.new(self,t)
    dbg.start{"ColumnTable:new()"}
    setmetatable(o, self)
    self.__index  = self
-   local width  = 80
-   if (getenv("TERM")) then
+   if (t.width) then
+      width = t.width
+   else
       width  = TermWidth()
    end
 
@@ -146,7 +145,7 @@ function M._clearEmptyColumns2D(self, tbl)
    end
 
    local numActiveCol = #movA
-   
+
    local tt = {}
 
    for irow = 1, #tbl do
@@ -214,8 +213,9 @@ function M._entry_width2(self, t, szA)
    for j = 1, sz do
       local a = t[j]
       local b = {}
-      for i = 1,#a do
-         b[i]    = {prt = length(a[i]),              wrt = a[i]:len()}
+      for i = 1,dim2 do
+         local entry = a[i] or ""
+         b[i]    = {prt = length(entry) or 0,        wrt = entry:len() or 0}
          minA[i] = {prt = min(minA[i].prt,b[i].prt), wrt = min(minA[i].wrt,b[i].wrt)}
          maxA[i] = {prt = max(maxA[i].prt,b[i].prt), wrt = max(maxA[i].wrt,b[i].wrt)}
       end
@@ -236,8 +236,8 @@ function M._entry_width2(self, t, szA)
    --dbg.printA{name = "minA", a = minA}
    --dbg.printA{name = "maxA", a = maxA}
 
-   local imin = {prt = iminPrt, wrt = iminWrt}
-   local imax = {prt = imaxPrt, wrt = imaxWrt}
+   imin = {prt = iminPrt, wrt = iminWrt}
+   imax = {prt = imaxPrt, wrt = imaxWrt}
    dbg.fini()
    return imin, imax
 end
@@ -253,7 +253,7 @@ function M._columnSum1(self, istart, iend)
    local szA     = self.szA
    local maxVprt = 0
    local maxVwrt = 0
-   
+
    for i = istart, iend do
       maxVprt = max(maxVprt, szA[i].prt)
       maxVwrt = max(maxVwrt, szA[i].wrt)
@@ -280,6 +280,7 @@ function M._columnSum2(self, istart, iend)
    for i = istart, iend do
       local a = szA[i]
       for idim = 1, dim2 do
+
          maxA[idim].prt = max(maxA[idim].prt, a[idim].prt)
          maxA[idim].wrt = max(maxA[idim].wrt, a[idim].wrt)
       end
@@ -334,9 +335,9 @@ function M._display2(self,i, icol)
    local sum      = 0
 
    local lastCol  = dim2
-   for i = dim2, 1, -1 do
-      if (widthA[i].prt > 0) then break end
-      lastCol = i - 1
+   for j = dim2, 1, -1 do
+      if (widthA[j].prt > 0) then break end
+      lastCol = j - 1
    end
 
    for idim = 1, dim2 do
@@ -429,7 +430,12 @@ function M._number_of_columns_rows(self,t)
 
       dbg.print{"ncols: ",ncols, " sumWrt: ",sumWrt," sumPrt: ",sumPrt,"\n"}
 
-      if (sumWrt < self.term_width) then
+      ----------------------------------------------------------------------
+      -- Use sumPrt to wrap the columns at the number of printed characters.
+      -- Use sumWrt to wrap the columns at the number of written characters.
+
+      local sum = sumPrt
+      if (sum < self.term_width) then
          results = ncols
          break
       end
@@ -438,7 +444,7 @@ function M._number_of_columns_rows(self,t)
    local istart = 1
    local iskip  = nrows - 1
    local ncols  = results or 1
-   local nrows  = math.ceil(sz/ncols)
+   nrows        = math.ceil(sz/ncols)
    dbg.print{"ncols: ",ncols,", nrows: ",nrows,"\n"}
 
    self.ncols   = math.ceil(sz/nrows)
