@@ -108,7 +108,7 @@ do
     local debug_upvalueid = debug and debug.upvalueid or nil
     local debug_upvaluejoin = debug and debug.upvaluejoin or nil
     local load    = (_VERSION == "Lua 5.1") and loadstring or load
-    local unpack  = unpack or table.unpack
+    local unpack  = (_VERSION == "Lua 5.1") and unpack or table.unpack
     local table   = table
     table.deepcopy_copyfunc_list = {
       --["type"] = function(stack, orig, copy, state, temp1, temp2, temp..., tempN)
